# -*- coding: utf-8 -*-
#
# File: PlominoAccessControl.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.com>"""
__docformat__ = 'plaintext'

from Products.PluginIndexes.common.UnIndex import UnIndex

class PlominoColumnIndex(UnIndex):

	"""Index for Plomino columns.
	"""

	meta_type="PlominoColumnIndex"

	query_options = ["query","range"]

	def index_object(self, documentId, obj, threshold=None):
		"""Index an object.

		'documentId' is the integer ID of the document.
		'obj' is the object to be indexed.
		'threshold' is the number of words to process between committing
		subtransactions.  If None, subtransactions are disabled.
		"""

		returnStatus = 0
		parentdb = self.getParentDatabase()
		doc = obj.__of__(parentdb)
		if(self.id.startswith("PlominoViewColumn_")):
			param = self.id.split('_')
			viewname=param[1]
			if not doc.isSelectedInView(viewname):
				return 0
			columnname=param[2]
			newValue = doc.computeColumnValue(viewname, columnname)
		else:
			return 0

		oldValue = self._unindex.get( documentId, None )
		if newValue != oldValue:
			if oldValue is not None:
				self.removeForwardIndexEntry(oldValue, documentId)
				if type(oldValue) is list:
					for kw in oldValue:
						self.removeForwardIndexEntry(kw, documentId)
				if newValue is None:
					try:
						del self._unindex[documentId]
					except ConflictError:
						raise
					except:
						pass
			if newValue is not None:
				self.insertForwardIndexEntry( newValue, documentId )
				if type(newValue) is list:
					for kw in newValue:
						self.insertForwardIndexEntry(kw, documentId)
				self._unindex[documentId] = newValue

			returnStatus = 1

		return returnStatus




# -*- coding: utf-8 -*-
#
# File: PlominoDesignManager.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.com>"""
__docformat__ = 'plaintext'

from Products.CMFPlomino.config import *

from Acquisition import *
from HttpUtils import authenticateAndLoadURL, authenticateAndPostToURL
from Products.PythonScripts.PythonScript import PythonScript
import re
from ZPublisher.HTTPResponse import HTTPResponse
from ZPublisher.HTTPRequest import HTTPRequest
from ZPublisher.HTTPRequest import FileUpload
from OFS.ObjectManager import ObjectManager
from Products.PageTemplates.ZopePageTemplate import manage_addPageTemplate
from Products.CMFCore.utils import getToolByName
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Persistence import Persistent
from webdav.Lockable import wl_isLocked
from xml.dom.minidom import getDOMImplementation
from xml.dom.minidom import parseString
import xmlrpclib
from cStringIO import StringIO
import traceback
import codecs
import os
import sys
import glob
import transaction

from migration.migration import migrate
from index.PlominoIndex import PlominoIndex
from exceptions import PlominoScriptException, PlominoDesignException
from PlominoUtils import asUnicode

# get AT specific schemas for each Plomino class
from Products.CMFPlomino.PlominoForm import schema as form_schema
from Products.CMFPlomino.PlominoAction import schema as action_schema
from Products.CMFPlomino.PlominoField import schema as field_schema
from Products.CMFPlomino.PlominoHidewhen import schema as hidewhen_schema
from Products.CMFPlomino.PlominoAgent import schema as agent_schema
from Products.CMFPlomino.PlominoView import schema as view_schema
from Products.CMFPlomino.PlominoColumn import schema as column_schema

plomino_schemas = {'PlominoForm': form_schema, 
                   'PlominoAction': action_schema,
                   'PlominoField': field_schema,
                   'PlominoHidewhen': hidewhen_schema,
                   'PlominoAgent': agent_schema,
                   'PlominoView': view_schema,
                   'PlominoColumn': column_schema
                   }
extra_schema_attributes = ['excludeFromNav']

import logging
logger = logging.getLogger('Plomino')


class PlominoDesignManager(Persistent):
    """Plomino design import/export features
    """
    security = ClassSecurityInfo()

    # Methods

    security.declareProtected(DESIGN_PERMISSION, 'refreshDB')
    def refreshDB(self, REQUEST=None):
        """all actions to take when reseting a DB (after import for instance)
        """
        logger.info('Refreshing database '+self.id)
        report = []

        self.setStatus("Refreshing design")

        # migrate to current version
        messages = migrate(self)
        for msg in messages:
            report.append(msg)
            logger.info(msg)

        #check folders
        if not hasattr(self, 'resources'):
            resources = Folder('resources')
            resources.title='resources'
            self._setObject('resources', resources)
        msg = 'Resources folder OK'
        report.append(msg)
        logger.info(msg)
        if not hasattr(self, 'scripts'):
            scripts = Folder('scripts')
            scripts.title='scripts'
            self._setObject('scripts', scripts)
        self.cleanFormulaScripts()
        msg = 'Scripts folder OK and clean'
        report.append(msg)
        logger.info(msg)

        # clean portal_catalog
        portal_catalog = self.portal_catalog
        catalog_entries = portal_catalog.search({'portal_type' : ['PlominoDocument'], 'path': '/'.join(self.getPhysicalPath())})
        for d in catalog_entries:
            portal_catalog.uncatalog_object(d.getPath())
        msg = 'Portal catalog clean'
        report.append(msg)
        logger.info(msg)

        # destroy the index
        self.manage_delObjects(self.getIndex().getId())
        msg = 'Old index removed'
        report.append(msg)
        logger.info(msg)

        #create new blank index
        index = PlominoIndex(FULLTEXT=self.FulltextIndex)
        self._setObject(index.getId(), index)
        self.getIndex().no_refresh = True
        msg = 'New index created'
        report.append(msg)
        logger.info(msg)

        #declare all the view formulas and columns index entries
        for v_obj in self.getViews():
            self.getIndex().createSelectionIndex('PlominoViewFormula_'+v_obj.getViewName())
            for c in v_obj.getColumns():
                v_obj.declareColumn(c.getColumnName(), c)
        for f_obj in self.getForms() :
            for f in f_obj.getFormFields() :
                if f.getToBeIndexed() :
                    self.getIndex().createFieldIndex(f.id, f.getFieldType())
        self.getIndex().no_refresh = False
        msg = 'Index structure initialized'
        report.append(msg)
        logger.info(msg)

        #reindex all the documents
        #documents = [d.getObject() for d in self.portal_catalog.search({'portal_type' : ['PlominoDocument'], 'path': '/'.join(self.getPhysicalPath())})]
        documents = self.getAllDocuments()
        total_docs = len(documents)
        msg = 'Existing documents: '+ str(total_docs)
        report.append(msg)
        logger.info(msg)
        total = 0
        counter = 0
        errors = 0
        self.setStatus("Re-indexing (0%)")
        txn = transaction.get()
        for d in documents:
            try:
                #self.getIndex().indexDocument(d)
                d.save(onSaveEvent=False)
                total = total + 1
            except Exception, e:
                errors = errors + 1
                logger.info("Ouch! \n%s\n%s" % (e, `d`))
            counter = counter + 1
            if counter == 10:
                self.setStatus("Re-indexing (%d%%)" % int(100*(total+errors)/total_docs))
                txn.savepoint(optimistic=True)
                counter = 0
                logger.info("%d documents re-indexed successfully, %d errors(s) ...(still running)" % (total, errors))
        self.setStatus("Ready")
        txn.commit()
        msg = "%d documents re-indexed successfully, %d errors(s)" % (total, errors)
        report.append(msg)
        logger.info(msg)

        # update Plone workflow state
        workflow_tool = getToolByName(self, 'portal_workflow')
        wfs = workflow_tool.getWorkflowsFor(self)
        for wf in wfs:
            if not isinstance( wf, DCWorkflowDefinition ):
                continue
            wf.updateRoleMappingsFor( self )
        msg = 'Plone workflow update'
        report.append(msg)
        logger.info(msg)

        if REQUEST:
            self.writeMessageOnPage(MSG_SEPARATOR.join(report), REQUEST, False)
            REQUEST.RESPONSE.redirect(self.absolute_url()+"/DatabaseDesign")

    security.declareProtected(DESIGN_PERMISSION, 'exportDesign')
    def exportDesign(self, targettype='file', targetfolder='', dbsettings=True, designelements=None, REQUEST=None, **kw):
        """ Export design elements to XML.
        The targettype can be file, server, or folder.
        """
        if REQUEST:
            entire=REQUEST.get('entire')
            targettype=REQUEST.get('targettype')
            targetfolder=REQUEST.get('targetfolder')
            dbsettings=REQUEST.get('dbsettings')

        if dbsettings == "Yes":
            dbsettings = True
        else:
            dbsettings = False

        if not designelements is None:
            if entire=="Yes":
                designelements = None
            else:
                designelements=REQUEST.get('designelements')
                if designelements:
                    if type(designelements)==str:
                        designelements=[designelements]

        if targettype in ["server", "file"]:
            xmlstring = self.exportDesignAsXML(elementids=designelements, dbsettings=dbsettings)

            if targettype == "server":
                if REQUEST:
                    targetURL=REQUEST.get('targetURL')
                    username=REQUEST.get('username')
                    password=REQUEST.get('password')
                result=authenticateAndPostToURL(targetURL+"/importDesignFromXML", username, password, 'exportDesignAsXML.xml', xmlstring)
                REQUEST.RESPONSE.redirect(self.absolute_url()+"/DatabaseDesign")
            elif targettype == "file":
                if REQUEST:
                    REQUEST.RESPONSE.setHeader('content-type', 'text/xml')
                    REQUEST.RESPONSE.setHeader("Content-Disposition", "attachment; filename="+self.id+".xml")
                return xmlstring
        elif targettype == "folder":
            if not designelements:
                designelements = ([o.id for o in self.getForms()] +
                                  [o.id for o in self.getViews()] +
                                  [o.id for o in self.getAgents()] +
                                  ["resources/"+id for id in self.resources.objectIds()])
            exportpath = os.path.join(targetfolder,(self.id))
            resources_exportpath = os.path.join(exportpath,('resources'))
            if os.path.isdir(exportpath):
                # remove previous export
                for f in glob.glob(os.path.join(exportpath,"*.xml")):
                    os.remove(f)
                if os.path.isdir(resources_exportpath):
                    for f in glob.glob(os.path.join(resources_exportpath,"*.xml")):
                        os.remove(f)
            else:
                os.makedirs(exportpath)
            if len([id for id in designelements if id.startswith('resources/')]) > 0:
                if not os.path.isdir(resources_exportpath):
                    os.makedirs(resources_exportpath)

            for id in designelements:
                if id.startswith('resources/'):
                    path = os.path.join(resources_exportpath, (id.split('/')[-1]+'.xml'))
                    xmlstring = self.exportDesignAsXML(elementids=[id], dbsettings=False)
                    self.saveFile(path, xmlstring)
                else:
                    path = os.path.join(exportpath, (id+'.xml'))
                    xmlstring = self.exportDesignAsXML(elementids=[id], dbsettings=False)
                    self.saveFile(path, xmlstring)
            if dbsettings:
                path = os.path.join(exportpath, ('dbsettings.xml'))
                xmlstring = self.exportDesignAsXML(elementids=[], dbsettings=True)
                self.saveFile(path, xmlstring)

    @staticmethod
    def saveFile(path, content):
        fileobj = codecs.open(path, "w", "utf-8")
        fileobj.write(content)
        fileobj.close()

    security.declareProtected(DESIGN_PERMISSION, 'importDesign')
    def importDesign(self, REQUEST=None):
        """import design elements in current database
        """
        submit_import=REQUEST.get('submit_import')
        entire=REQUEST.get('entire')
        sourcetype=REQUEST.get('sourcetype')
        sourceURL=REQUEST.get('sourceURL')
        username=REQUEST.get('username')
        password=REQUEST.get('password')
        replace_design=REQUEST.get('replace_design')
        replace = replace_design == "Yes"
        if submit_import:
            if sourcetype=="server":
                export_url=sourceURL+"/exportDesignAsXML"
                if not entire=="Yes":
                    designelements=REQUEST.get('designelements')
                    if designelements:
                        if type(designelements)==str:
                            designelements=[designelements]
                        export_url=export_url+"?elementids="+"@".join(designelements)
                xmlstring=authenticateAndLoadURL(export_url, username, password).read()
                self.importDesignFromXML(xmlstring, replace=replace)

            elif sourcetype =="folder":
                path = REQUEST.get('sourcefolder')
                self.importDesignFromXML(from_folder=path, replace=replace)

            else:
                fileToImport = REQUEST.get('sourceFile',None)
                if not fileToImport:
                    raise PlominoDesignException, 'file required'
                if not isinstance(fileToImport, FileUpload):
                    raise PlominoDesignException, 'unrecognized file uploaded'
                xmlstring=fileToImport.read()
                self.importDesignFromXML(xmlstring, replace=replace)

            no_refresh_documents = REQUEST.get('no_refresh_documents', 'No')
            if no_refresh_documents == 'No':
                self.refreshDB()
            REQUEST.RESPONSE.redirect(self.absolute_url()+"/DatabaseDesign")
        else:
            REQUEST.RESPONSE.redirect(self.absolute_url()+"/DatabaseDesign?username="+username+"&password="+password+"&sourceURL="+sourceURL)

    security.declareProtected(DESIGN_PERMISSION, 'getViewsList')
    def getViewsList(self):
        """return the database views ids in a string
        """
        views = self.getViews()
        ids = ""
        for v in views:
            ids=ids+v.id+"/"
        return ids

    security.declareProtected(DESIGN_PERMISSION, 'getFormsList')
    def getFormsList(self):
        """return the database forms ids in a string
        """
        forms = self.getForms()
        ids = ""
        for f in forms:
            ids=ids+f.id+"/"
        return ids

    security.declareProtected(DESIGN_PERMISSION, 'getAgentsList')
    def getAgentsList(self):
        """return the database agents ids in a string
        """
        agents = self.getAgents()
        ids = ""
        for a in agents:
            ids=ids+a.id+"/"
        return ids

    security.declareProtected(DESIGN_PERMISSION, 'getResourcesList')
    def getResourcesList(self):
        """return the database resources objects ids in a string
        """
        res = self.resources.objectIds()
        ids = ""
        for i in res:
            ids=ids+i+"/"
        return ids

    security.declareProtected(DESIGN_PERMISSION, 'getRemoteViews')
    def getRemoteViews(self, sourceURL, username, password):
        """get views ids list from remote database
        """
        views = authenticateAndLoadURL(sourceURL+"/getViewsList", username, password).read()
        ids = views.split('/')
        ids.pop()
        return ids

    security.declareProtected(DESIGN_PERMISSION, 'getRemoteForms')
    def getRemoteForms(self, sourceURL, username, password):
        """get forms ids list from remote database
        """
        forms = authenticateAndLoadURL(sourceURL+"/getFormsList", username, password).read()
        ids = forms.split('/')
        ids.pop()
        return ids

    security.declareProtected(DESIGN_PERMISSION, 'getRemoteAgents')
    def getRemoteAgents(self, sourceURL, username, password):
        """get agents ids list from remote database
        """
        agents = authenticateAndLoadURL(sourceURL+"/getAgentsList", username, password).read()
        ids = agents.split('/')
        ids.pop()
        return ids

    security.declareProtected(DESIGN_PERMISSION, 'getRemoteResources')
    def getRemoteResources(self, sourceURL, username, password):
        """get resources ids list from remote database
        """
        res = authenticateAndLoadURL(sourceURL+"/getResourcesList", username, password).read()
        ids = res.split('/')
        ids.pop()
        return ['resources/'+i for i in ids]

    security.declarePublic('getFormulaScript')
    def getFormulaScript(self, script_id):
        if hasattr(self.scripts, script_id):
            ps=getattr(self.scripts, script_id)
            return ps
        else:
            return None

    security.declarePublic('cleanFormulaScripts')
    def cleanFormulaScripts(self, script_id_pattern=None):
        for s in self.scripts.objectIds():
            if script_id_pattern is None:
                self.scripts._delObject(s)
            elif s.startswith(script_id_pattern):
                self.scripts._delObject(s)

    security.declarePublic('compileFormulaScript')
    def compileFormulaScript(self, script_id, formula, with_args=False):
        ps = self.getFormulaScript(script_id)
        if ps is None:
            ps=PythonScript(script_id)
            self.scripts._setObject(script_id, ps)
        ps = self.getFormulaScript(script_id)

        if with_args:
            ps._params="*args"
        str_formula="plominoContext = context\n"
        str_formula=str_formula+"plominoDocument = context\n"
        str_formula=str_formula+"from Products.CMFPlomino.PlominoUtils import "+SAFE_UTILS+'\n'

        r = re.compile('#Plomino import (.+)[\r\n]')
        for i in r.findall(formula):
            scriptname=i.strip()
            try:
                script_code = str(self.resources._getOb(scriptname))
            except:
                script_code = "#ALERT: "+scriptname+" not found in resources"
            formula = formula.replace('#Plomino import '+scriptname, script_code)

        if formula.strip().count('\n')>0:
            str_formula=str_formula+formula
        else:
            if formula.startswith('return '):
                str_formula=str_formula+formula
            else:
                str_formula=str_formula+"return "+formula
        ps.write(str_formula)
        if self.debugMode:
            logger.info(script_id + " compiled")
        return ps

    security.declarePublic('runFormulaScript')
    def runFormulaScript(self, script_id, context, formula_getter, with_args=False, *args):
#        if self.debugMode:
#            logger.info('Evaluating '+script_id+' with context '+str(context))
        try:
            ps = self.getFormulaScript(script_id)
            if ps is None:
                ps = self.compileFormulaScript(script_id, formula_getter(), with_args)
                if self.debugMode and hasattr(ps, 'errors') and ps.errors:
                    logger.info('ps.errors : ' + str(ps.errors))
            contextual_ps=ps.__of__(context)
            result = None
            if with_args:
                result = contextual_ps(*args)
            else:
                result = contextual_ps()
            if self.debugMode and hasattr(contextual_ps, 'errors') and contextual_ps.errors:
                logger.info('python errors at '+str(context)+' in '+script_id+': ' + str(contextual_ps.errors))
            return result
        except Exception, e:
            raise PlominoScriptException(context, e, formula_getter, script_id)
#            msg = self.traceErr(e, context, script_id, formula_getter)
#            raise PlominoScriptException(context.absolute_url_path(), formula_getter, message=msg)

    security.declarePrivate('traceRenderingErr')
    def traceRenderingErr(self, e, context):
        """trace rendering errors
        """
        if self.debugMode:
            #traceback
            formatted_lines = traceback.format_exc().splitlines()
            msg = "\r\n".join(formatted_lines[-3:]).strip()
            #code / value
            msg = msg + "Plomino rendering error with context : " + str(context)            
        else:
            msg = None

        if msg:
            logger.error(msg)

    security.declarePublic('callScriptMethod')
    def callScriptMethod(self, scriptname, methodname, *args):
        id="script_"+scriptname+"_"+methodname
        try:
            script_code = str(self.resources._getOb(scriptname))
        except:
            script_code = "#ALERT: "+scriptname+" not found in resources"
        formula=lambda:script_code+'\n\nreturn '+methodname+'(*args)'

        return self.runFormulaScript(id, self, formula, True, *args)

    security.declarePublic('writeMessageOnPage')
    def writeMessageOnPage(self, infoMsg, REQUEST, error = False):
        """adds portal message        
        """
        #message
        plone_tools = getToolByName(self, 'plone_utils')
        if plone_tools is not None:
            #portal message type
            if error:
                msgType = 'error'
            else:
                msgType = 'info'

            #split message   
            infoMsg = infoMsg.split(MSG_SEPARATOR)

            #display it
            for msg in infoMsg:
                if msg:
                    plone_tools.addPortalMessage(msg, msgType, REQUEST)

    security.declarePublic('getRenderingTemplate')
    def getRenderingTemplate(self, templatename, request=None):
        """
        """
        skin=self.portal_skins.cmfplomino_templates
        if hasattr(skin, templatename):
            pt = getattr(skin, templatename)
            if request:
                pt.REQUEST = request
            else:
                request = getattr(pt, 'REQUEST', None)
                proper_request = request and pt.REQUEST.__class__.__name__=='HTTPRequest'
                if not proper_request:
                    # XXX What *else* could REQUEST be here?
                    # we are not in an actual web context, but we a need a
                    # request object to have the template working
                    response = HTTPResponse(stdout=sys.stdout)
                    env = {'SERVER_NAME':'fake_server',
                           'SERVER_PORT':'80',
                           'REQUEST_METHOD':'GET'}
                    pt.REQUEST = HTTPRequest(sys.stdin, env, response)

            # we also need a RESPONSE
            if not pt.REQUEST.has_key('RESPONSE'):
                pt.REQUEST['RESPONSE']=HTTPResponse()

            return pt
        else:
            return None

    security.declareProtected(DESIGN_PERMISSION, 'exportDesignAsXML')
    def exportDesignAsXML(self, elementids=None, REQUEST=None, dbsettings=True):
        """
        """
        impl = getDOMImplementation()
        doc = impl.createDocument(None, "plominodatabase", None)
        root = doc.documentElement
        root.setAttribute("id", self.id)

        if REQUEST:
            str_elementids=REQUEST.get("elementids")
            if str_elementids is not None:
                elementids = str_elementids.split("@")

        if elementids is None:
            elements = self.getForms() + self.getViews() + self.getAgents() + [o for o in self.resources.getChildNodes()] 
        else:
            elements = []
            for id in elementids:
                if id.startswith('resources/'):
                    e = getattr(self.resources, id.split('/')[1])
                else:
                    e = getattr(self, id)
                elements.append(e)

        # Sort elements by type (to store forms before views), then by id
        elements.sort(key=lambda elt: elt.getId())
        elements.sort(key=lambda elt: elt.Type())

        designNode = doc.createElement('design')

        # export database settings
        if dbsettings:
            node = self.exportElementAsXML(doc, self, isDatabase=True)
            designNode.appendChild(node)

        # export database design elements
        for e in elements:
            if e.meta_type in plomino_schemas.keys():
                node = self.exportElementAsXML(doc, e)
            else:
                node = self.exportResourceAsXML(doc, e)

            designNode.appendChild(node)

        root.appendChild(designNode)
        s = doc.toxml()

        if REQUEST:
            REQUEST.RESPONSE.setHeader('content-type', "text/xml;charset=utf-8")

        # Usage of lxml to make a pretty output
        try:
            from lxml import etree
            parser = etree.XMLParser(strip_cdata=False, encoding="utf-8")
            return etree.tostring(etree.XML(s, parser), encoding="utf-8", pretty_print=True)
        except ImportError:
            return s.encode('utf-8').replace("><", ">\n<")

    security.declareProtected(DESIGN_PERMISSION, 'exportElementAsXML')
    def exportElementAsXML(self, xmldoc, obj, isDatabase=False):
        """
        """
        if isDatabase:
            node = xmldoc.createElement('dbsettings')
            schema = sys.modules[self.__module__].schema
        else:
            node = xmldoc.createElement('element')
            node.setAttribute('id', obj.id)
            node.setAttribute('type', obj.Type())
            title = obj.title
            node.setAttribute('title', title)
            schema = plomino_schemas[obj.Type()]

        for f in schema.fields():
            fieldNode = xmldoc.createElement(f.getName())
            field_type = f.getType()
            fieldNode.setAttribute('type', field_type)
            v = f.get(obj)
            if v is not None:
                if field_type=="Products.Archetypes.Field.TextField":
                    text = xmldoc.createCDATASection(f.getRaw(obj).decode('utf-8'))
                else:
                    text = xmldoc.createTextNode(str(f.get(obj)))
                fieldNode.appendChild(text)
            node.appendChild(fieldNode)

        # add AT standard extra attributes
        for extra in extra_schema_attributes:
            f = obj.Schema().getField(extra)
            if f is not None:
                fieldNode = xmldoc.createElement(extra)
                field_type = f.getType()
                fieldNode.setAttribute('type', field_type)
                v = f.get(obj)
                if v is not None:
                    if field_type=="Products.Archetypes.Field.TextField":
                        text = xmldoc.createCDATASection(f.getRaw(obj).decode('utf-8'))
                    else:
                        text = xmldoc.createTextNode(str(f.get(obj)))
                    fieldNode.appendChild(text)
                node.appendChild(fieldNode)
#        
        if obj.Type() == "PlominoField":
            adapt = obj.getSettings()
            if adapt is not None:
                items = {}
                for k in adapt.parameters.keys():
                    if hasattr(adapt, k):
                        items[k] = adapt.parameters[k]
                #items = dict(adapt.parameters)
                if len(items)>0:
                    # export field settings
                    str_items = xmlrpclib.dumps((items,), allow_none=1)
                    dom_items = parseString(str_items)
                    node.appendChild(dom_items.documentElement)
        if not isDatabase:
            elementslist = obj.objectIds()
            if len(elementslist)>0:
                elements = xmldoc.createElement('elements')
                for id in elementslist:
                    elementNode = self.exportElementAsXML(xmldoc, getattr(obj, id))
                    elements.appendChild(elementNode)
                node.appendChild(elements)

        if isDatabase:
           acl = xmldoc.createElement('acl')
           acl.setAttribute('AnomynousAccessRight', obj.AnomynousAccessRight)
           acl.setAttribute('AuthenticatedAccessRight', obj.AuthenticatedAccessRight)
           str_UserRoles = xmlrpclib.dumps((obj.UserRoles,), allow_none=1)
           dom_UserRoles = parseString(str_UserRoles)
           dom_UserRoles.firstChild.setAttribute('id', 'UserRoles')
           acl.appendChild(dom_UserRoles.documentElement)
           str_SpecificRights = xmlrpclib.dumps((obj.getSpecificRights(),), allow_none=1)
           dom_SpecificRights = parseString(str_SpecificRights)
           dom_SpecificRights.firstChild.setAttribute('id', 'SpecificRights')
           acl.appendChild(dom_SpecificRights.documentElement)
           node.appendChild(acl)

        return node

    security.declareProtected(DESIGN_PERMISSION, 'exportResourceAsXML')
    def exportResourceAsXML(self, xmldoc, obj):
        """
        """
        node = xmldoc.createElement('resource')
        id = obj.id
        if callable(id):
            id=id()
        node.setAttribute('id', id)
        resource_type = obj.meta_type
        node.setAttribute('type', resource_type)
        node.setAttribute('title', obj.title)
        if resource_type=="Page Template":
            data = xmldoc.createCDATASection(obj.read())
        else:
            node.setAttribute('contenttype', obj.getContentType())
            stream = obj.data
            if not hasattr(stream, "encode"):
                stream = stream.data
            data = xmldoc.createCDATASection(stream.encode('base64'))
        node.appendChild(data)
        return node

    security.declareProtected(DESIGN_PERMISSION, 'importDesignFromXML')
    def importDesignFromXML(self, xmlstring=None, REQUEST=None, from_folder=None, replace=False):
        """
        """
        logger.info("Start design import")
        self.setStatus("Importing design")
        self.getIndex().no_refresh = True
        txn = transaction.get()
        xml_strings = []
        count = 0
        total = 0
        if from_folder:
            if not os.path.isdir(from_folder):
                raise PlominoDesignException, '%s does not exist' % path
            xml_files = (glob.glob(os.path.join(from_folder, '*.xml')) +
                     glob.glob(os.path.join(from_folder, 'resources/*.xml')))
            total_elements = len(xml_files)
            for p in xml_files:
                fileobj = codecs.open(p, 'r', 'utf-8')
                xml_strings.append(fileobj.read())
        else:
            if REQUEST:
                f=REQUEST.get("file")
                xml_strings.append(f.read())
            else:
                xml_strings.append(xmlstring)
            total_elements = None

        if replace:
            logger.info("Replace mode: removing current design")
            designelements = [o.id for o in self.getForms()] \
                                 + [o.id for o in self.getViews()] \
                                 + [o.id for o in self.getAgents()]
            ObjectManager.manage_delObjects(self, designelements)
            ObjectManager.manage_delObjects(self.resources, self.resources.objectIds())
            logger.info("Current design removed")

        for xmlstring in xml_strings:
            xmlstring = xmlstring.replace(">\n<", "><")
            xmldoc = parseString(xmlstring)
            design = xmldoc.getElementsByTagName("design")[0]
            elements = [e for e in design.childNodes
                            if e.nodeName in ('resource', 'element', 'dbsettings')]

            if not total_elements:
                total_elements = len(elements)

            e = design.firstChild
            while e is not None:
                name = str(e.nodeName)
                if name in ('resource', 'element', 'dbsettings'):
                    if name == 'dbsettings':
                        logger.info("Import db settings")
                        self.importDbSettingsFromXML(e)
                    if name == 'element':
                        logger.info("Import "+e.getAttribute('id'))
                        self.importElementFromXML(self, e)
                    if name == 'resource':
                        logger.info("Import resource "+e.getAttribute('id'))
                        self.importResourceFromXML(self.resources, e)
                    count = count + 1
                    total = total + 1
                if count == 10:
                    self.setStatus("Importing design (%d%%)" % int(100*total/total_elements))
                    logger.info("(%d elements committed, still running...)" % total)
                    txn.savepoint(optimistic=True)
                    count = 0
                e = e.nextSibling

        logger.info("(%d elements imported)" % total)
        self.setStatus("Ready")
        txn.commit()
        self.getIndex().no_refresh = False

    security.declareProtected(DESIGN_PERMISSION, 'importElementFromXML')
    def importElementFromXML(self, container, node):
        """
        """
        id = node.getAttribute('id')
        element_type = node.getAttribute('type')
        if id in container.objectIds():
            ob = getattr(container, id)
            if wl_isLocked(ob):
                ob.wl_clearLocks()
            container.manage_delObjects([id])
        container.invokeFactory(element_type, id=id)
#        if not(hasattr(container, id)):
#            container.invokeFactory(element_type, id=id)
        obj = getattr(container, id)
        if obj.Type() == element_type:
            # note: there might be an existing object with the same id but with a
            # different type, in this case, we do not import
            title = node.getAttribute('title')
            obj.setTitle(title)
            child = node.firstChild
            at_values = {}
            settings_values = {}
            while child is not None:
                name = child.nodeName
                if name == 'id':
                    pass
                elif name == 'elements':
                    # current object is a form or a view, it contains sub-objects
                    # (fields, actions, columns, hide-when)
                    subchild = child.firstChild
                    while subchild is not None:
                        if subchild.nodeType == subchild.ELEMENT_NODE:
                            self.importElementFromXML(obj, subchild)
                        subchild = subchild.nextSibling
                elif name == 'params':
                    # current object is a field, the params tag contains the
                    # specific settings
                    result, method = xmlrpclib.loads(node.toxml().encode('utf-8'))
                    parameters = result[0]
                    for key in parameters.keys():
                        v = parameters[key]
                        if v is not None:
                            if hasattr(v, 'encode'):
                                v = unicode(v)
                            else:
                                if hasattr(v, 'append'):
                                    uv = []
                                    for e in v:
                                        if hasattr(e, 'encode'):
                                            uv.append(unicode(e))
                                        else:
                                            uv.append(e)
                                    v = uv
                            settings_values[key] = v
                else:
                    if child.hasChildNodes():
                        field = obj.Schema().getField(name)

                        # Get cdata content if available, else get text node
                        cdatas = [n for n in child.childNodes if n.nodeType == n.CDATA_SECTION_NODE]
                        if len(cdatas) > 0:
                            v = cdatas[0].data
                        else:
                            v = child.firstChild.data
                        v = v.strip()
                        at_values[name] = v
                child = child.nextSibling

            if len(at_values) > 0:
                obj.processForm(REQUEST=None, values=at_values)
            if len(settings_values) > 0:
                adapt = obj.getSettings()
                for key in settings_values.keys():
                    setattr(adapt, key, settings_values[key])

    security.declareProtected(DESIGN_PERMISSION, 'importDbSettingsFromXML')
    def importDbSettingsFromXML(self, node):
        """
        """
        child = node.firstChild
        while child is not None:
            name = child.nodeName
            if name=='acl':
                anonymousaccessright = child.getAttribute('AnomynousAccessRight')
                self.AnomynousAccessRight = anonymousaccessright
                self.setPlominoPermissions("Anonymous", anonymousaccessright)
                authenticatedaccessright = child.getAttribute('AuthenticatedAccessRight')
                self.AuthenticatedAccessRight = authenticatedaccessright
                self.setPlominoPermissions("Authenticated", authenticatedaccessright)
                subchild = child.firstChild
                while subchild is not None:
                    if subchild.nodeType == subchild.ELEMENT_NODE:
                        result, method = xmlrpclib.loads(subchild.toxml())
                        object_type = subchild.getAttribute('id')
                        if object_type=="SpecificRights":
                            self.specific_rights = result[0]
                        else:
                            self.UserRoles = result[0]
                    subchild = subchild.nextSibling

            else:
                if child.hasChildNodes():
                    field = self.Schema().getField(name)
                    #field.set(self, child.firstChild.data)
                    result=field.widget.process_form(self, field, {name : child.firstChild.data})
                    field.set(self, result[0])
            child = child.nextSibling

    security.declareProtected(DESIGN_PERMISSION, 'importResourceFromXML')
    def importResourceFromXML(self, container, node):
        """
        """
        id = str(node.getAttribute('id'))
        resource_type = node.getAttribute('type')
        if resource_type == "Page Template":
            if not(hasattr(container, id)):
                obj = manage_addPageTemplate(container, id)
            else:
                obj = getattr(container, id)
            obj.title = node.getAttribute('title')
            obj.write(node.firstChild.data)
        else:
            if not(hasattr(container, id)):
                container.manage_addFile(id)
            obj = getattr(container, id)
            obj.meta_type = resource_type
            obj.title = node.getAttribute('title')
            obj.update_data(node.firstChild.data.decode('base64'), content_type=node.getAttribute('contenttype'))






# -*- coding: utf-8 -*-
#
# File: richtext.py
#
# Copyright (c) 2009 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.com>"""
__docformat__ = 'plaintext'

from zope.formlib import form
from zope.interface import implements
from zope.schema import getFields
from zope.schema import TextLine, Text, Choice
from zope.schema.vocabulary import SimpleVocabulary

from dictionaryproperty import DictionaryProperty

from Products.Five.formlib.formbase import EditForm

from base import IBaseField, BaseField

class IRichtextField(IBaseField):
    """
    Text field schema
    """
    height = TextLine(title=u'Height',
                      description=u'Height in pixels',
                      required=False)

class RichtextField(BaseField):
    """
    """
    implements(IRichtextField)

for f in getFields(IRichtextField).values():
    setattr(RichtextField, f.getName(), DictionaryProperty(f, 'parameters'))

class SettingForm(EditForm):
    """
    """
    form_fields = form.Fields(IRichtextField)




# -*- coding: utf-8 -*-
#
# File: PlominoView.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# Generator: ArchGenXML Version 2.0
#            http://plone.org/products/archgenxml
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from zope.interface import implements
import interfaces
from Products.ATContentTypes.content.folder import ATFolder
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.CMFPlomino.config import *
from exceptions import PlominoScriptException
from validator import isValidPlominoId

from AccessControl import Unauthorized
import csv, cStringIO
from Acquisition import aq_inner
import PlominoDocument

import simplejson as json

schema = Schema((

    StringField(
        name='id',
        widget=StringField._properties['widget'](
            label="Id",
            description="If changed after creation, database refresh is needed",
            label_msgid='CMFPlomino_label_view_id',
            description_msgid='CMFPlomino_help_view_id',
            i18n_domain='CMFPlomino',
        ),
        validators = ("isValidId", isValidPlominoId),
    ),
    TextField(
        name='SelectionFormula',
        widget=TextAreaWidget(
            label="Selection formula",
            description="The view selection formula is a line of Python code which should return True or False. The formula will be evaluated for each document in the database to decide if the document must be displayed in the view or not. plominoDocument is a reserved name in formulae, it returns the current Plomino document. Each document field value can be accessed directly as an attribute of the document: plominoDocument.<fieldname>",
            label_msgid='CMFPlomino_label_SelectionFormula',
            description_msgid='CMFPlomino_help_SelectionFormula',
            i18n_domain='CMFPlomino',
        ),
        default = "True",
    ),
    StringField(
        name='SortColumn',
        widget=StringField._properties['widget'](
            label="Sort column",
            description="Column used to sort the view",
            label_msgid='CMFPlomino_label_SortColumn',
            description_msgid='CMFPlomino_help_SortColumn',
            i18n_domain='CMFPlomino',
        ),
        schemata="Sorting",
    ),
    BooleanField(
        name='Categorized',
        default="0",
        widget=BooleanField._properties['widget'](
            label="Categorized",
            description="Categorised on first column",
            label_msgid='CMFPlomino_label_Categorized',
            description_msgid='CMFPlomino_help_Categorized',
            i18n_domain='CMFPlomino',
        ),
        schemata="Sorting",
    ),
    TextField(
        name='FormFormula',
        widget=TextAreaWidget(
            label="Form formula",
            description="Documents open from the view will use the form defined by the following formula (they use their own form if empty)",
            label_msgid='CMFPlomino_label_FormFormula',
            description_msgid='CMFPlomino_help_FormFormula',
            i18n_domain='CMFPlomino',
        ),
    ),
    BooleanField(
        name='ReverseSorting',
        default="0",
        widget=BooleanField._properties['widget'](
            label="Reverse sorting",
            description="Reverse sorting",
            label_msgid='CMFPlomino_label_ReverseSorting',
            description_msgid='CMFPlomino_help_ReverseSorting',
            i18n_domain='CMFPlomino',
        ),
        schemata="Sorting",
    ),
    StringField(
        name='ActionBarPosition',
        default="TOP",
        widget=SelectionWidget(
            label="Position of the action bar",
            description="Select the position of the action bar",
            label_msgid='CMFPlomino_label_ActionBarPosition',
            description_msgid='CMFPlomino_help_ActionBarPosition',
            i18n_domain='CMFPlomino',
        ),
#        schemata="Parameters",
        vocabulary=[["TOP", "At the top of the page"], ["BOTTOM", "At the bottom of the page"], ["BOTH", "At the top and at the bottom of the page "]],
    ),
    BooleanField(
        name='HideDefaultActions',
        default="0",
        widget=BooleanField._properties['widget'](
            label="Hide default actions",
            description="Delete, Close actions will not be displayed in the action bar",
            label_msgid='CMFPlomino_label_HideViewDefaultActions',
            description_msgid='CMFPlomino_help_HideViewDefaultActions',
            i18n_domain='CMFPlomino',
        ),
#        schemata="Parameters",
    ),
    BooleanField(
        name='HideInMenu',
        default="0",
        widget=BooleanField._properties['widget'](
            label="Hide in menu",
            description="It will not appear in the database main menu",
            label_msgid='CMFPlomino_label_HideInMenu',
            description_msgid='CMFPlomino_help_HideInMenu',
            i18n_domain='CMFPlomino',
        ),
#        schemata="Parameters",
    ),
    StringField(
        name='Widget',
        default="BASIC",
        widget=SelectionWidget(
            label="Widget",
            description="Rendering mode",
            label_msgid='CMFPlomino_label_ViewWidget',
            description_msgid='CMFPlomino_help_ViewWidget',
            i18n_domain='CMFPlomino',
        ),
        vocabulary= [["BASIC", "Basic html"], ["DYNAMICTABLE", "Dynamic table"]],
#        schemata="Parameters",
    ),
    TextField(
        name='DynamicTableParameters',
        widget=TextAreaWidget(
            label="Dynamic Table Parameters",
            description="Change these options to customize the dynamic table.",
            label_msgid='CMFPlomino_label_DynamicTableParameters',
            description_msgid='CMFPlomino_help_DynamicTableParameters',
            i18n_domain='CMFPlomino',
        ),
        default=u"""
'bPaginate': false,
'bLengthChange': false,
'bFilter': true,
'bSort': true,
'bInfo': true,
'bAutoWidth': false""",
#        schemata="Parameters",
    ),
    StringField(
        name='ViewTemplate',
        widget=StringField._properties['widget'](
            label="View template",
            description="Leave blank to use default",
            label_msgid='CMFPlomino_label_ViewTemplate',
            description_msgid='CMFPlomino_help_ViewTemplate',
            i18n_domain='CMFPlomino',
        ),
#        schemata="Parameters",
    ),
    IntegerField(
        name='Position',
        widget=IntegerField._properties['widget'](
            label="Position",
            label_msgid="CMFPlomino_label_Position",
            description="Position in menu",
            description_msgid="CMFPlomino_description_Position",
            i18n_domain='CMFPlomino',
        ),
#        schemata="Parameters",
    ),
),
)

PlominoView_schema = getattr(ATFolder, 'schema', Schema(())).copy() + \
    schema.copy()

class PlominoView(ATFolder):
    """
    """
    security = ClassSecurityInfo()
    implements(interfaces.IPlominoView)

    meta_type = 'PlominoView'
    _at_rename_after_creation = False

    schema = PlominoView_schema

    # Methods

    security.declarePublic('checkBeforeOpenView')
    def checkBeforeOpenView(self):
        """check read permission and open view NOTE: if READ_PERMISSION set
        on the 'view' actionb itself, it causes error 'maximum recursion
        depth exceeded' if user hasn't permission
        """
        if self.checkUserPermission(READ_PERMISSION):
            if not self.getViewTemplate()=="":
                pt=self.resources._getOb(self.getViewTemplate())
                return pt.__of__(self)()
            else:
                return self.OpenView()
        else:
            raise Unauthorized, "You cannot read this content"

    security.declarePublic('getViewName')
    def getViewName(self):
        """Get view name
        """
        return self.id

    security.declarePublic('getParentDatabase')
    def getParentDatabase(self):
        """Get parent database
        """
        return self.getParentNode()

    def __bobo_traverse__(self, request, name):
        if self.documents.has_key(name):
            return aq_inner(getattr(self.documents, name)).__of__(self)
        return BaseObject.__bobo_traverse__(self, request, name)

    security.declarePublic('getAllDocuments')
    def getAllDocuments(self, start=1, limit=0, only_allowed=True):
        """Get all documents (as CatalogBrains, you have to use getObject()
        to get the real PlominoDocument)
        """
        index = self.getParentDatabase().getIndex()
        sortindex = self.getSortColumn()
        if sortindex=='':
            sortindex=None
        else:
            sortindex=self.getIndexKey(sortindex)
        results=index.dbsearch({'PlominoViewFormula_'+self.getViewName() : True}, sortindex, self.getReverseSorting(), only_allowed=only_allowed)
        if limit==0:
            return results
        else:
            return results[start-1:start-1+limit]

    security.declarePublic('getColumns')
    def getColumns(self):
        """Get colums
        """
        columnslist = self.portal_catalog.search({'portal_type' : ['PlominoColumn'], 'path': '/'.join(self.getPhysicalPath())}, sort_index='getObjPositionInParent')
        #orderedcolumns = []
        #for c in columnslist:
        #    c_obj = c.getObject()
        #    if not(c_obj is None):
        #        orderedcolumns.append([c_obj.Position, c_obj])
        #orderedcolumns.sort()
        #return [i[1] for i in orderedcolumns]
        return [c.getObject() for c in columnslist]

    security.declarePublic('getActions')
    def getActions(self, target, hide=True):
        """Get actions
        """
        all = self.portal_catalog.search({'portal_type' : ['PlominoAction'], 'path': '/'.join(self.getPhysicalPath())})

        filtered = []
        for a in all:
            obj_a=a.getObject()
            if hide:
                try:
                    #result = RunFormula(target, obj_a.getHidewhen())
                    result = self.runFormulaScript("action_"+obj_a.getParentNode().id+"_"+obj_a.id+"_hidewhen", target, obj_a.Hidewhen)
                except PlominoScriptException, e:
                    e.reportError('"%s" action hide-when failed' % obj_a.Title())
                    #if error, we hide anyway
                    result = True
                if not result:
                    filtered.append(obj_a)
            else:
                filtered.append(obj_a)
        return filtered

    security.declarePublic('getColumn')
    def getColumn(self,column_name):
        """Get a single column
        """
        return getattr(self, column_name)

    security.declarePublic('evaluateViewForm')
    def evaluateViewForm(self,doc):
        """compute the form to be used to open documents
        """
        try:
            #result = RunFormula(doc, self.getFormFormula())
            result = self.runFormulaScript("view_"+self.id+"_formformula", doc, self.FormFormula)
        except PlominoScriptException, e:
            e.reportError('"%s" form formula failed' % self.Title())
            result = ""
        return result

    security.declarePublic('at_post_edit_script')
    def at_post_edit_script(self):
        db = self.getParentDatabase()
        self.cleanFormulaScripts("view_"+self.id)
        if not db.DoNotReindex:
            self.getParentDatabase().getIndex().refresh()

    security.declarePublic('at_post_create_script')
    def at_post_create_script(self):
        """post create
        """
        db = self.getParentDatabase()
        db.getIndex().createSelectionIndex('PlominoViewFormula_'+self.getViewName())
        if not db.DoNotReindex:
            self.getParentDatabase().getIndex().refresh()

    security.declarePublic('declareColumn')
    def declareColumn(self,column_name,column_obj):
        """declare column
        """
        db = self.getParentDatabase()
        refresh = not(db.DoNotReindex)

        if column_obj.Formula:
            db.getIndex().createIndex('PlominoViewColumn_'+self.getViewName()+'_'+column_name, refresh=refresh)
        else:
            fieldpath = column_obj.SelectedField.split('/')
            form = self.getParentDatabase().getForm(fieldpath[0])
            if form:
                field = form.getFormField(fieldpath[1])
                if field:
                    field.setToBeIndexed(True)
                    field.at_post_edit_script()
                    #db.getIndex().createFieldIndex(field.id, field.getFieldType())
                else:
                    column_obj.setFormula("'Non-existing field'")
                    db.getIndex().createIndex('PlominoViewColumn_'+self.getViewName()+'_'+column_name, refresh=refresh)
            else:
                column_obj.setFormula("'Non-existing form'")
                db.getIndex().createIndex('PlominoViewColumn_'+self.getViewName()+'_'+column_name, refresh=refresh)

    security.declarePublic('getCategorizedColumnValues')
    def getCategorizedColumnValues(self,column_name):
        """return existing values for the given key and add the empty value
        """
        alldocs = self.getAllDocuments()
        allvalues = [getattr(doc, self.getIndexKey(column_name)) for doc in alldocs]
        categories={}
        for itemvalue in allvalues:
            if type(itemvalue)==list:
                for v in itemvalue:
                    if not(v in categories):
                        categories[v]=1
                    else:
                        categories[v]+=1
            else:
                if itemvalue is not None:
                    if not(itemvalue in categories):
                        categories[itemvalue]=1
                    else:
                        categories[itemvalue]+=1
        uniquevalues = categories.keys()
        uniquevalues.sort()
        return [(v, categories[v]) for v in uniquevalues]


    security.declarePublic('getCategoryViewEntries')
    def getCategoryViewEntries(self,category_column_name,category_value):
        """get category view entry
        """
        index = self.getParentDatabase().getIndex()
        sortindex = self.getSortColumn()
        if sortindex=='':
            sortindex=None
        else:
            sortindex=self.getIndexKey(sortindex)

        return index.dbsearch(
            {
                'PlominoViewFormula_'+self.getViewName() : True,
                self.getIndexKey(category_column_name) : category_value
            },
            sortindex,
            self.getReverseSorting())

    security.declarePublic('getColumnSums')
    def getColumnSums(self):
        """return the sum of non null values for each column
        marked as summable
        """
        sums = {}
        docs = self.getAllDocuments()
        for col in self.getColumns():
            if col.DisplaySum:
                indexkey = self.getIndexKey(col.getColumnName())
                values = [getattr(doc, indexkey) for doc in docs]
                try:
                    s = sum([v for v in values if v is not None])
                except:
                    s = 0
                sums[col.id] = s
        return sums

    security.declareProtected(READ_PERMISSION, 'exportCSV')
    def exportCSV(self, REQUEST=None, displayColumnsTitle='False', separator="\t", brain_docs = None, quotechar='"', quoting=csv.QUOTE_NONNUMERIC):
        """export columns values as CSV
        IMPORTANT : brain_docs are supposed to be ZCatalog brains
        """
        if type(quoting) is str:
            #convert to int when passed via querystring
            try:
                quoting = int(quoting)
            except:
                quoting=csv.QUOTE_NONNUMERIC

        if brain_docs is None:
            docs = self.getAllDocuments()
        else:
            docs = brain_docs
        result=""
        columns=[c.id for c in self.getColumns()]
        vname=self.getViewName()

        stream = cStringIO.StringIO()
        writer = csv.writer(stream, delimiter=separator, quotechar=quotechar, quoting=quoting)

        # add column titles
        if displayColumnsTitle=='True' :
            titles = [c.title for c in self.getColumns()]
            writer.writerow(titles)

        for doc in docs:
            values=[]
            for cname in columns:
                v=getattr(doc, self.getIndexKey(cname))
                if v is None:
                    v=''
                elif isinstance(v, basestring):
                    v = v.encode('utf-8')
                else:
                    v = unicode(v).encode('utf-8')
                values.append(v)
            writer.writerow(values)

        if REQUEST:
            REQUEST.RESPONSE.setHeader('content-type', 'text/csv; charset=utf-8')
            REQUEST.RESPONSE.setHeader("Content-Disposition", "attachment; filename="+self.id+".csv")
        return stream.getvalue()

    security.declareProtected(READ_PERMISSION, 'exportXLS')
    def exportXLS(self, REQUEST, displayColumnsTitle='False', brain_docs = None):
        """export columns values in an HTML table
        and set content-type to launch Excel
        IMPORTANT : brain_docs are supposed to be ZCatalog brains
        """
        if brain_docs is None:
            docs = self.getAllDocuments()
        else:
            docs = brain_docs
        result=""
        columns=[c.id for c in self.getColumns()]
        vname=self.getViewName()

        rows = []

        # add column titles
        if displayColumnsTitle=='True' :
            titles = [c.title.encode('utf-8') for c in self.getColumns()]
            rows.append(titles)

        for doc in docs:
            values=[]
            for cname in columns:
                v=getattr(doc, self.getIndexKey(cname))
                if v is None:
                    v=''
                elif isinstance(v, basestring):
                    v = v.encode('utf-8')
                else:
                    v = unicode(v).encode('utf-8')
                values.append(v)
            rows.append(values)

        html ="""<html><head>
    <meta http-equiv="Content-Type"
          content="text/html;charset=utf-8" />
<body><table>"""
        for row in rows:
            html = html + "<tr>" + ''.join(["<td>%s</td>" % v for v in row]) + "</tr>\n"

        html = html + "</table>\n</body></html>"
        REQUEST.RESPONSE.setHeader('content-type', 'application/vnd.ms-excel; charset=utf-8')
        REQUEST.RESPONSE.setHeader("Content-Disposition", "inline; filename="+self.id+".xls")
        return html


    security.declarePublic('getPosition')
    def getPosition(self):
        """Return the view position in the database
        """
        try :
            return self.Position
        except Exception :
            return None

    security.declarePublic('getDocumentsByKey')
    def getDocumentsByKey(self, key):
        """Get documents which sorted column value match the given key
        """
        index = self.getParentDatabase().getIndex()
        sortindex = self.getSortColumn()
        if sortindex=='':
            return []
        else:
            sortindex=self.getIndexKey(sortindex)
            results=index.dbsearch({'PlominoViewFormula_'+self.getViewName() : True,
                                    sortindex : key
                                    },
                                   sortindex, self.getReverseSorting())
        return [d.getObject() for d in results]

    security.declarePublic('tojson')
    def tojson(self):
        """Returns a JSON representation of view data 
        """
        data = []
        columnids = [col.id for col in self.getColumns() if not getattr(col, 'HiddenColumn', False)]
        for doc in self.getAllDocuments():
            row = [doc.getPath().split('/')[-1]]
            for colid in columnids:
                v = getattr(doc, self.getIndexKey(colid), '')
                if not isinstance(v, str):
                    v = unicode(v).encode('utf-8')
                row.append(v or '&nbsp;')

            data.append(row)
        return json.dumps({ 'aaData': data })

    security.declarePublic('getIndexKey')
    def getIndexKey(self, columnName):
        """Returns an index key depending of which one exists.
        It can be a computed index (PlominoViewColumn_) or a link to a column. 
        """
        key = 'PlominoViewColumn_%s_%s' % (self.getViewName(), columnName)
        if key not in self.getParentDatabase().plomino_index.Indexes:
            fieldPath = self.getColumn(columnName).SelectedField.split('/')
            if len(fieldPath) > 1:
                key = fieldPath[1]
            else:
                key = ''
        return key


registerType(PlominoView, PROJECTNAME)
# end of class PlominoView






# -*- coding: utf-8 -*-
#
# File: PlominoAccessControl.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.com>"""
__docformat__ = 'plaintext'

from Products.ZCTextIndex.ZCTextIndex import ZCTextIndex

class PlominoFileIndex(ZCTextIndex):

	"""Index for Plomino files attachments.
	"""

	meta_type="PlominoFileIndex"

	query_options = ["query"]

	def index_object(self, documentId, obj, threshold=None):
	    """Wrapper for  index_doc()  handling indexing of multiple attributes.

	    Enter the document with the specified documentId in the index
	    under the terms extracted from the indexed text attributes,
	    each of which should yield either a string or a list of
	    strings (Unicode or otherwise) to be passed to index_doc().
	    """
	    # XXX We currently ignore subtransaction threshold

	    res = 0
	    all_texts = []
	    if self._fieldname.startswith("PlominoFiles_"):
	    	attr=self._fieldname[13:]
	        text = str(obj.getRenderedItem(attr,form=None, convertattachments=True))
	        if text:
	            if isinstance(text, (list, tuple, )):
	                all_texts.extend(text)
	            else:
	                all_texts.append(text)

	    # Check that we're sending only strings
	    all_texts = filter(lambda text: isinstance(text, basestring), \
	                       all_texts)
	    if all_texts:
	        return self.index.index_doc(documentId, all_texts)
	    return res



import os.path
from zope.component import getMultiAdapter

def get_language_path(context):
    """Select the right language file from the context language. 
    """
    
    if not getattr(context, 'REQUEST', None):
        return 'No language'
    portal_state = getMultiAdapter((context, context.REQUEST), name=u'plone_portal_state')
    current_language = portal_state.language()
    
    lang = current_language.split('-')[0]
    
    dir, _ = os.path.split(os.path.abspath(__file__))
    if not os.path.isfile(os.path.join(dir, "lang", lang + ".txt")):
        lang = "en"
    
    return context.portal_url() + "/++resource++dataTables.lang/" + lang + ".txt"

class DataTables(object):
    """ Provides access to get_language_path via a browserview.
    """
    
    def __init__(self, context, request):
        """Initialize adapter."""
        self.context = context
        self.request = request
    
    def getLanguagePath(self):
        """
        """
        
        context = self.context.aq_inner
        return get_language_path(context)



# -*- coding: utf-8 -*-
#
# File: events.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Xavier PERROT <xavier.perrot@makina-corpus.com>"""
__docformat__ = 'plaintext'

from Products.CMFPlomino.config import *
from Products.CMFPlomino.PlominoDocument import PlominoDocument

def PlominoDocumentRemoveEventHandler(obj, event):
    """remove event handler for plomino documents
    comments from plone/app/linkintegrity/handlers.py
    """
    # if the object the event was fired on doesn't have a `REQUEST` attribute
    # we can safely assume no direct user action was involved and therefore
    # never raise a link integrity exception...
    # (this should also fix http://plone.org/products/cachefu/issues/86)
    #if not hasattr(obj, 'REQUEST'):
    #    return

    # since the event gets called for every subobject before it's
    # called for the item deleted directly via _delObject (event.object)
    # itself, but we do not want to present the user with a confirmation
    # form for every (referred) subobject, so we remember and skip them...
    #if obj is not event.object:
    #    return

    #TODO : test if deletion confirmation has been done, and unindex document
    #TODO : find how to know if confirmation has been done
    #return
    pass



from AccessControl import ClassSecurityInfo

security = ClassSecurityInfo()

PROJECTNAME = 'CMFPlomino'

VERSION = '1.9'

security.declarePublic('ADD_DESIGN_PERMISSION')
ADD_DESIGN_PERMISSION = 'CMFPlomino: Add Plomino design elements'
security.declarePublic('ADD_CONTENT_PERMISSION')
ADD_CONTENT_PERMISSION = 'CMFPlomino: Add Plomino content'
security.declarePublic('READ_PERMISSION')
READ_PERMISSION = 'CMFPlomino: Read documents'
security.declarePublic('EDIT_PERMISSION')
EDIT_PERMISSION = 'CMFPlomino: Edit documents'
security.declarePublic('CREATE_PERMISSION')
CREATE_PERMISSION = 'CMFPlomino: Create documents'
security.declarePublic('REMOVE_PERMISSION')
REMOVE_PERMISSION = 'CMFPlomino: Remove documents'
security.declarePublic('DESIGN_PERMISSION')
DESIGN_PERMISSION = 'CMFPlomino: Modify Database design'
security.declarePublic('ACL_PERMISSION')
ACL_PERMISSION = 'CMFPlomino: Control Database ACL'

SAFE_UTILS='DateToString, StringToDate, DateRange, sendMail, userFullname, userInfo, htmlencode, Now, asList, urlencode, csv_to_array, MissingValue, open_url, asUnicode, array_to_csv'

FIELD_TYPES={"TEXT": ["Text", "FieldIndex"],
             "NUMBER": ["Number", "FieldIndex"],
             #"FLOAT": ["Float", "FieldIndex"],
             "RICHTEXT": ["Rich text", "ZCTextIndex"],
             "DATETIME": ["Date/Time", "DateIndex"],
             "NAME": ["Name", "FieldIndex"],
             #"NAMES": ["Names", "ZCTextIndex"],
             "SELECTION": ["Selection list", "KeywordIndex"],
             #"MULTISELECTION": ["Multi-Selection list", "KeywordIndex"],
             #"CHECKBOX": ["Check boxes", "KeywordIndex"],
             #"RADIO": ["Radio buttons", "ZCTextIndex"],
             "ATTACHMENT": ["File attachment", "ZCTextIndex"],
             "DOCLINK": ["Doclink", "KeywordIndex"],
             "GOOGLECHART": ["Google chart", "FieldIndex"],
             "GOOGLEVISUALIZATION": ["Google visualization", "FieldIndex"],
             #"DATAGRID": ["Datagrid", "ZCTextIndex"],
             }

FIELD_MODES = [["EDITABLE", "Editable"], ["COMPUTED", "Computed"], ["CREATION", "Computed on creation"], ["DISPLAY", "Computed for display"], ["COMPUTEDONSAVE", "Computed on save"],]

#FCKeditor configuration styles
FCK_STYLES = '''
<Style name="Heading" element="h2" />
<Style name="Subheading" element="h3" />
<Style name="Literal" element="pre" />
<Style name="Discreet paragraph" element="p">
  <Attribute name="class" value="discreet" />
</Style>
<Style name="Pull-quote" element="div">
  <Attribute name="class" value="pullquote" />
</Style>
<Style name="Call-out" element="p">
  <Attribute name="class" value="callout" />
</Style>
<Style name="Highlight" element="span">
  <Attribute name="class" value="visualHighlight" />
</Style>
<Style name="Heading cell" element="th" />
<Style name="Plomino Field" element="span">
  <Attribute name="class" value="plominoFieldClass" />
</Style>
<Style name="Plomino Action" element="span">
  <Attribute name="class" value="plominoActionClass" />
</Style>
<Style name="Plomino Hide-when formula" element="span">
  <Attribute name="class" value="plominoHidewhenClass" />
</Style>
<Style name="Plomino Subform" element="span">
  <Attribute name="class" value="plominoSubformClass" />
</Style>'''

MSG_SEPARATOR = '\n'



# -*- coding: utf-8 -*-
#
# File: datagrid.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.com>"""
__docformat__ = 'plaintext'

from zope.formlib import form
from zope.interface import implements
from zope import component
from zope.pagetemplate.pagetemplatefile import PageTemplateFile

from zope.schema import getFields
from zope.schema import Text, TextLine
from zope.schema.vocabulary import SimpleVocabulary

import simplejson as json

from Products.Five.formlib.formbase import EditForm

from Products.CMFPlomino.PlominoUtils import csv_to_array

from Products.CMFPlomino.interfaces import IPlominoField
from Products.CMFPlomino.fields.dictionaryproperty import DictionaryProperty

from Products.CMFPlomino.fields.base import IBaseField, BaseField

from Products.CMFPlomino.browser.javascript.dataTables.utils import get_language_path

class IDatagridField(IBaseField):
    """
    Text field schema
    """
    associated_form = TextLine(title=u'Associated form',
                description=u'Form to use to create/edit rows',
                required=False)
    
    field_mapping = TextLine(title=u'Columns/fields mapping',
                description=u'Field ids from the associated form, ordered as the columns, separated by commas',
                required=False)
        
    jssettings = Text(title=u'Javascript settings',
                      description=u'jQuery datatable parameters',
                      default=u"""
"aoColumns": [
    { "sTitle": "Column 1" },
    { "sTitle": "Column 2", "sClass": "center" }
],
"bPaginate": false,
"bLengthChange": false,
"bFilter": false,
"bSort": false,
"bInfo": false,
"bAutoWidth": false
""",
                      required=False)

class DatagridField(BaseField):
    """
    """
    implements(IDatagridField)
    
    plomino_field_parameters = {'interface': IDatagridField,
                                'label': "Datagrid",
                                'index_type': "ZCTextIndex"}
    
    read_template = PageTemplateFile('datagrid_read.pt')
    edit_template = PageTemplateFile('datagrid_edit.pt')
    
    def getParameters(self):
        """
        """
        return self.jssettings
    
    def processInput(self, submittedValue):
        """
        """
        try:
            return json.loads(submittedValue)
        except:
            return []
    
    def tojson(self, value):
        """
        """
        if value is None or value == "":
            value = []
        if isinstance(value, basestring):
            return value
        return json.dumps(value)
    
    def getLang(self):
        """
        """
        return get_language_path(self.context)
    
    def getFieldValue(self, form, doc, editmode, creation, request):
        """
        """
        fieldValue = BaseField.getFieldValue(self, form, doc, editmode, creation, request)
        
        mode = self.context.getFieldMode()
                        
        if not(mode=="EDITABLE" and editmode):
             # fieldValue is a array of arrays, where we must replace raw values with
             # rendered values
             try:
                 child_form_id = self.associated_form
                 if child_form_id is not None:
                     db = self.context.getParentDatabase()
                     child_form = db.getForm(child_form_id)
                     fields = self.field_mapping.split(',')
                     fields_obj = [child_form.getFormField(f) for f in fields]
                     # avoid bad field ids
                     fields_obj = [f for f in fields_obj if f is not None]
                     fields_to_render = [f.id for f in fields_obj if f.getFieldType() not in ["DATETIME", "NUMBER", "TEXT", "RICHTEXT"]]
                     
                     rendered_values = []
                     for row in fieldValue:
                         row_values = {}
                         j = 0
                         for v in row:
                             if fields[j] in fields_to_render:
                                 row_values[fields[j]] = v
                             j = j + 1
                         if len(row_values) > 0:
                             row_values['Plomino_Parent_Document'] = doc.id 
                             tmp = TemporaryDocument(db, child_form, row_values)
                             tmp.setItem('Form', child_form_id)
                         rendered_row = []
                         i = 0
                         for f in fields:
                             if f in fields_to_render:
                                 rendered_row.append(tmp.getRenderedItem(f))
                             else:
                                 rendered_row.append(row[i])
                             i = i + 1
                         rendered_values.append(rendered_row)
                     fieldValue = rendered_values
             except:
                 pass
        
        mapping = self.field_mapping
        if mapping:
            col_number = len(mapping.split(','))
            resized = []
            empty = ['']
            for row in fieldValue:
                if len(row) < col_number:
                    resized.append(row + empty * (col_number - len(row)))
                elif len(row) > col_number:
                    resized.append(row[:col_number])
                else:
                    resized.append(row)
            fieldValue = resized
        return fieldValue

component.provideUtility(DatagridField, IPlominoField, 'DATAGRID')

for f in getFields(IDatagridField).values():
    setattr(DatagridField, f.getName(), DictionaryProperty(f, 'parameters'))

class SettingForm(EditForm):
    """
    """
    form_fields = form.Fields(IDatagridField)
    



# -*- coding: utf-8 -*-
#
# File: datetime.py
#
# Copyright (c) 2009 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.com>"""
__docformat__ = 'plaintext'

from zope.formlib import form
from zope.interface import implements
from zope.schema import getFields
from zope.schema import TextLine, Text, Choice
from zope.schema.vocabulary import SimpleVocabulary

from dictionaryproperty import DictionaryProperty

from Products.Five.formlib.formbase import EditForm
from Products.CMFPlomino.PlominoUtils import StringToDate

from base import IBaseField, BaseField

class IDatetimeField(IBaseField):
    """
    DateTime field schema
    """
    format = TextLine(title=u'Format',
                      description=u'Date/time format (if different than database default format)',
                      required=False)
    startingyear = TextLine(title=u'Starting year',
                      description=u'Oldest year selectable in the calendar widget',
                      required=False)

class DatetimeField(BaseField):
    """
    """
    implements(IDatetimeField)

    def validate(self, submittedValue):
        """
        """
        errors=[]
        fieldname = self.context.id
        submittedValue = submittedValue.strip()
        try:
            # calendar widget default format is '%Y-%m-%d %H:%M' and might use the AM/PM format
            if submittedValue[-2:] in ['AM', 'PM']:
                v = StringToDate(submittedValue, '%Y-%m-%d %I:%M %p')
            else:
                v = StringToDate(submittedValue, '%Y-%m-%d %H:%M')
        except:
            errors.append(fieldname+" must be a date/time (submitted value was: "+submittedValue+")")
        return errors

    def processInput(self, submittedValue):
        """
        """
        submittedValue = submittedValue.strip()
        # calendar widget default format is '%Y-%m-%d %H:%M' and might use the AM/PM format
        if submittedValue[-2:] in ['AM', 'PM']:
            d = StringToDate(submittedValue, '%Y-%m-%d %I:%M %p')
        else:
            d = StringToDate(submittedValue, '%Y-%m-%d %H:%M')
        return d

    def getFieldValue(self, form, doc, editmode, creation, request):
        """
        """
        fieldValue = BaseField.getFieldValue(self, form, doc, editmode, creation, request)

        mode = self.context.getFieldMode()

        if mode=="EDITABLE":
            if doc is None and not(creation) and request is not None:
                fieldValue = request.get(fieldName, '')
                if not(fieldValue=='' or fieldValue is None):
                    fieldValue = StringToDate(fieldValue, form.getParentDatabase().getDateTimeFormat())
        return fieldValue

for f in getFields(IDatetimeField).values():
    setattr(DatetimeField, f.getName(), DictionaryProperty(f, 'parameters'))

class SettingForm(EditForm):
    """
    """
    form_fields = form.Fields(IDatetimeField)




# -*- coding: utf-8 -*-
#
# File: datetime.py
#
# Copyright (c) 2009 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.com>"""
__docformat__ = 'plaintext'


def afterFieldModified(obj, event):
    """
    """
    obj.cleanFormulaScripts("field_"+obj.getPhysicalPath()[-2]+"_"+obj.id)


# -*- coding: utf-8 -*-
#
# File: setuphandlers.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# Generator: ArchGenXML Version 2.0
#            http://plone.org/products/archgenxml
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'


import logging
logger = logging.getLogger('CMFPlomino: setuphandlers')
from Products.CMFPlomino.config import PROJECTNAME
from Products.CMFPlomino.config import DEPENDENCIES
import os
from Products.CMFCore.utils import getToolByName
import transaction
##code-section HEAD
from Products.CMFPlomino.config import FCK_STYLES
##/code-section HEAD

def isNotCMFPlominoProfile(context):
    return context.readDataFile("CMFPlomino_marker.txt") is None



def updateRoleMappings(context):
    """after workflow changed update the roles mapping. this is like pressing
    the button 'Update Security Setting' and portal_workflow"""
    if isNotCMFPlominoProfile(context): return 
    shortContext = context._profile_path.split(os.path.sep)[-3]
    if shortContext != 'CMFPlomino': # avoid infinite recursions
        return
    wft = getToolByName(context.getSite(), 'portal_workflow')
    wft.updateRoleMappings()

def postInstall(context):
    """Called as at the end of the setup process. """
    if isNotCMFPlominoProfile(context): return
    shortContext = context._profile_path.split(os.path.sep)[-3]
    if shortContext != 'CMFPlomino': # avoid infinite recursions
        return
    site = context.getSite()

    #run kupu customisation script
    if hasattr(site.portal_properties, 'kupu_library_tool'):
        logger.info('run Kupu Customisation Script')
        skin=site.portal_skins.cmfplomino_styles
        script = getattr(skin,'kupu-customisation-policy', None)
        if script:
            script()
            logger.info('runKupuCustomisationScript done')
        else:
            logger.info('runKupuCustomisationScript : kupu-customisation-policy not found')

    #customise FCKedior configuration
    if hasattr(site.portal_properties, 'fckeditor_properties'):
        properties = site.portal_properties.fckeditor_properties
        styles = getattr(properties, 'fck_menu_styles') + FCK_STYLES
        properties.manage_changeProperties(fck_menu_styles = styles)

    # tidy up old jquery reference
    jsregistry = getToolByName(site, 'portal_javascripts')
    jsregistry.unregisterResource('++resource++plomino.javascript/jquery-v1.4.2.js')

##code-section FOOT
##/code-section FOOT



from zope.interface import directlyProvides
from Products.BTreeFolder2.BTreeFolder2 import manage_addBTreeFolder
from Products.CMFPlone.interfaces import IHideFromBreadcrumbs

from Products.CMFPlomino.fields.text import ITextField
from Products.CMFPlomino.fields.selection import ISelectionField
from Products.CMFPlomino.fields.number import INumberField
from Products.CMFPlomino.fields.datetime import IDatetimeField
from Products.CMFPlomino.fields.name import INameField
from Products.CMFPlomino.fields.doclink import IDoclinkField

import logging
logger = logging.getLogger('Plomino migration')

def migrate(db):
    """
    """
    messages = []
    if not(hasattr(db, "plomino_version")):
        msg = migrate_to_130(db)
        messages.append(msg)
    if db.plomino_version=="1.3.0":
        # no migration needed here
        db.plomino_version = "1.4.0"
    if db.plomino_version=="1.4.0":
        msg = migrate_to_15(db)
        messages.append(msg)
    if db.plomino_version=="1.5":
        msg = migrate_to_16(db)
        messages.append(msg)
    if db.plomino_version=="1.6":
        msg = migrate_to_161(db)
        messages.append(msg)
    if db.plomino_version=="1.6.1":
        # no migration needed here
        db.plomino_version = "1.6.2"
    if db.plomino_version=="1.6.2":
        # no migration needed here
        db.plomino_version = "1.6.3"
    if db.plomino_version=="1.6.3":
        msg = migrate_to_164(db)
        messages.append(msg)
    if db.plomino_version=="1.6.4":
        msg = migrate_to_17(db)
        messages.append(msg)
    if db.plomino_version=="1.7":
        msg = migrate_to_173(db)
        messages.append(msg)
    if db.plomino_version=="1.7.3":
        # no migration needed here
        db.plomino_version = "1.7.4"
    if db.plomino_version=="1.7.4":
        msg = migrate_to_175(db)
        messages.append(msg)
    if db.plomino_version=="1.7.5":
        # no migration needed here
        db.plomino_version = "1.8"
    if db.plomino_version=="1.8":
        # no migration needed here
        db.plomino_version = "1.9"
    return messages

def migrate_to_130(db):
    """ PlominoField schema has been changed, all type-specific parameters
    have been removed and are now handled through specific adapters.
    """
    db.setDebugMode(False)
    msg = "Migration to 1.3: DebugMode attribute added"

    for form in db.getForms():
        for field in form.getFormFields():
            type = field.getFieldType()
            if type == "TEXT":
                adapt = ITextField(field)
                adapt.widget = "TEXT"
            elif type == "NUMBER":
                adapt = INumberField(field)
                adapt.type = "INTEGER"
            elif type == "FLOAT":
                field.setFieldType("NUMBER")
                adapt = INumberField(field)
                adapt.type = "FLOAT"
            elif type == "NAME":
                adapt = INameField(field)
                adapt.type = "SINGLE"
            elif type == "NAMES":
                field.setFieldType("NAME")
                adapt = INameField(field)
                adapt.type = "MULTI"
            elif type in ("SELECTION", "MULTISELECTION", "CHECKBOX", "RADIO"):
                field.setFieldType("SELECTION")
                adapt = ISelectionField(field)
                l = getattr(field, "SelectionList", None)
                if l is not None:
                    adapt.selectionlist = list(l)
                v = getattr(field, "SelectionListFormula", None)
                if v is not None:
                    adapt.selectionlistformula = v.raw
                adapt.separator = getattr(field, "DisplayModList", getattr(field, "OtherDisplayMod", None))
                if type == "SELECTION":
                    adapt.widget = "SELECT"
                elif type == "MULTISELECTION":
                    adapt.widget = "MULTISELECT"
                elif type == "CHECKBOX":
                    adapt.widget = "CHECKBOX"
                elif type == "RADIO":
                    adapt.widget = "RADIO"
            elif type == "DOCLINK":
                adapt = IDoclinkField(field)
                v = getattr(field, "SelectionListFormula", None)
                if v is not None:
                    adapt.documentslistformula = v.raw
    msg = msg + ", FieldType remapped" 
    db.plomino_version = "1.3.0"
    return msg

def migrate_to_15(db):
    """ new attribute in Column: DisplaySum
    """
    for v_obj in db.getViews():
        for c in v_obj.getColumns():
            c.setDisplaySum(False)
    msg = "Migration to 1.5: DisplaySum attribute added"
    db.plomino_version = "1.5"
    return msg

def migrate_to_16(db):
    """ attribute Position in column is replaced by AtFolder sorting
    """
    for v_obj in db.getViews():
        # sort columns by their Position
        orderedcolumns = []
        for c in v_obj.getColumns():
            if not(c is None):
                orderedcolumns.append([c.Position, c])
        orderedcolumns.sort()

        # set the position using the previous sorting
        for i, c in enumerate(orderedcolumns):
            v_obj.moveObject(c[1].id, i)
            v_obj.plone_utils.reindexOnReorder(v_obj)

    msg = "Migration to 1.6: Position column attribute deleted"
    db.plomino_version = "1.6"
    return msg

def migrate_to_161(db):
    """ Fulltext index db attribute.
    """
    db.setFulltextIndex(False)
    msg = "Migration to 1.6.1: Fulltext index db attribute set to False"
    db.plomino_version = "1.6.1"
    return msg

def migrate_to_164(db):
    """ Fulltext index db attribute.
    """
    db.setDoNotReindex(False)
    msg = "Migration to 1.6.4: DoNotReindex db attribute set to False"
    db.plomino_version = "1.6.4"
    return msg

def migrate_to_17(db):
    """ dynamic hidewhen attribute
    """
    #for form in db.getForms():
    #    form.setIsDynamicHidewhen(False)
    msg = "Migration to 1.7: Dynamic hide-when initialized"
    db.plomino_version = "1.7"
    return msg

def migrate_to_173(db):
    """ dynamic hidewhen attribute moved
    """
    for form in db.getForms():
        for hidewhen in form.getHidewhenFormulas():
            hidewhen.setIsDynamicHidewhen(False)
    msg = "Migration to 1.7.3: Dynamic hide-when initialized"
    db.plomino_version = "1.7.3"
    return msg

def migrate_to_175(db):
    """ documents stores in BTreeFolder
    """
    manage_addBTreeFolder(db, id='plomino_documents')
    directlyProvides(db.plomino_documents, IHideFromBreadcrumbs)
    docids = [id for id in db.objectIds() if getattr(db, id).portal_type == "PlominoDocument"]
    cookie = db.manage_cutObjects(ids=docids)
    db.plomino_documents.manage_pasteObjects(cookie)
    msg = "Migration to 1.7.5: Documents moved in BTreeFolder"
    db.plomino_version = "1.7.5"
    return msg



# -*- coding: utf-8 -*-
#
# File: PlominoReplicationManager.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Xavier PERROT <xavier.perrot@makina-corpus.com>"""
__docformat__ = 'plaintext'

from Globals import Persistent
from Products.CMFPlomino.config import *

from Products.CMFPlomino.exceptions import PlominoReplicationException
try:
    from Products.ZpCron.crontab import CronTab
    from Products.ZpCron.Product import ZpCron
except:
    pass
import StringIO

class PlominoScheduler(Persistent):
    """Plomino scheduler features
    """
    security = ClassSecurityInfo()


    security.declarePrivate('managePlominoCronTab')
    def managePlominoCronTab(self, onDelete = False):
        """
        delete / set the cron configuration for 
        - agents
        - base replication
        """

        #ZpCron installed
        try:
            cronObj = self.Control_Panel.ZpCron
        except:
            cronObj = None

        if (cronObj):
            #buid hashmap
            tabCron = self.parseCronTab(cronObj)

            #base url
            baseUrl = self.absolute_url(1)

            #del commands for currrent base
            newTabCron = {}
            for cmd in tabCron:
                if not(baseUrl in cmd):
                    newTabCron[cmd] = tabCron[cmd]

            #rebuild base agents commands
            for agent in self.getAgents():
                if (agent.getScheduled()) and not(onDelete and agent.id == self.id):
                    userAgent = self.acl_users.absolute_url(1) + ':' + agent.getAgentUser()
                    if not userAgent.startswith('/'):
                        userAgent = '/' + userAgent
                    newTabCron[agent.absolute_url(1) + '/runAgent'] = {'user':userAgent,'cron':agent.getCron(),}

            #rebuild replication commands
            replications = self.getReplications()
            for replicationId in replications:
                replication = self.getReplication(replicationId)
                if replication['scheduled']:
                    username = self.acl_users.absolute_url(1) + ':' + replication['username']
                    if not username.startswith('/'):
                        username = '/' + username
                    newTabCron[baseUrl + '/replicate?replicationId='+replicationId] = {'user':username,'cron':replication['cron'],}

            #write file
            fileContent = ''
            for cmd in newTabCron:
                if cmd.startswith('/'):
                    command = cmd
                else:
                    command = '/' + cmd
                fileContent = fileContent + 'USER='+newTabCron[cmd]['user'] + '\n'
                fileContent = fileContent + newTabCron[cmd]['cron'] + ' ' + command + '\n'

            #save file
            self.Control_Panel.ZpCron.crontab = fileContent
            self.Control_Panel.ZpCron.update()

    security.declarePrivate('parseCronTab')
    def parseCronTab(self,cronObj):
        """
        buid a hashmap representing file
        key : action url (command)
        value : date, user, cron, params
        """
        #initialization
        res = {}
        currentUser = ''

        #file content
        crontab = cronObj.crontab

        #lines
        for line in crontab.split('\n'):
            #strip spaces
            line = line.strip()
            #no line
            if (not line):
                continue
            #empty line
            elif (line == '') or (len(line) == 0):
                continue
            #comments
            elif (line[0] in ('#', ';')):
                continue
            #@ line -> not managed
            elif (line[0] == '@'):
                continue
            # first caract is letter
            elif line[0].isalpha():
                var, user = line.split('=', 1)
                var = var.strip()
                user = user.strip()
                if user:
                    if user.startswith('"') and user.endswith('"'):
                        user = user[1:-1]
                    currentUser = user
            # first caract is * or num -> cron line
            elif line[0].isdigit() or line[0]=='*':
                line = line.split(None, 5)
                #user
                res[line[5]]={'user':currentUser,
                              'cron':' '.join(line[:5]),}
        return res



# -*- coding: utf-8 -*-
#
# File: PlominoField.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# Generator: ArchGenXML Version 2.0
#            http://plone.org/products/archgenxml
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT and Caroline ANSALDI <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from zope.interface import implements
import interfaces

from Products.CMFPlomino import fields

from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.CMFPlomino.config import *
from PlominoDocument import TemporaryDocument

##code-section module-header #fill in your manual code here
from Products.CMFPlomino.PlominoUtils import StringToDate
from fields.selection import ISelectionField
from fields.text import ITextField
from fields.datetime import IDatetimeField
from fields.name import INameField
from fields.doclink import IDoclinkField
from ZPublisher.HTTPRequest import FileUpload
from zope import component

##/code-section module-header

schema = Schema((

    StringField(
        name='id',
        widget=StringField._properties['widget'](
            label="Id",
            description="The field id",
            label_msgid='CMFPlomino_label_field_id',
            description_msgid='CMFPlomino_help_field_id',
            i18n_domain='CMFPlomino',
        ),
    ),
    StringField(
        name='FieldType',
        default="TEXT",
        widget=SelectionWidget(
            label="Field type",
            description="The kind of this field",
            label_msgid='CMFPlomino_label_FieldType',
            description_msgid='CMFPlomino_help_FieldType',
            i18n_domain='CMFPlomino',
        ),
        vocabulary='type_vocabulary',
    ),
    StringField(
        name='FieldMode',
        default="EDITABLE",
        widget=SelectionWidget(
            label="Field mode",
            description="How content will be generated",
            label_msgid='CMFPlomino_label_FieldMode',
            description_msgid='CMFPlomino_help_FieldMode',
            i18n_domain='CMFPlomino',
        ),
        vocabulary= FIELD_MODES,
    ),
    TextField(
        name='Formula',
        widget=TextAreaWidget(
            label="Formula",
            description="How to calculate field content",
            label_msgid='CMFPlomino_label_FieldFormula',
            description_msgid='CMFPlomino_help_FieldFormula',
            i18n_domain='CMFPlomino',
            rows=10,
        ),
    ),
    StringField(
        name='FieldReadTemplate',
        widget=StringField._properties['widget'](
            label="Field read template",
            description="Custom rendering template in read mode",
            label_msgid='CMFPlomino_label_FieldReadTemplate',
            description_msgid='CMFPlomino_help_FieldReadTemplate',
            i18n_domain='CMFPlomino',
        ),
    ),
    StringField(
        name='FieldEditTemplate',
        widget=StringField._properties['widget'](
            label="Field edit template",
            description="Custom rendering template in edit mode",
            label_msgid='CMFPlomino_label_FieldEditTemplate',
            description_msgid='CMFPlomino_help_FieldEditTemplate',
            i18n_domain='CMFPlomino',
        ),
    ),

    BooleanField(
        name='Mandatory',
        default="0",
        widget=BooleanField._properties['widget'](
            label="Mandatory",
            description="Is this field mandatory? (empty value will not be allowed)",
            label_msgid='CMFPlomino_label_FieldMandatory',
            description_msgid='CMFPlomino_help_FieldMandatory',
            i18n_domain='CMFPlomino',
        ),
    ),
    TextField(
        name='ValidationFormula',
        widget=TextAreaWidget(
            label="Validation formula",
            description="Evaluate the input validation",
            label_msgid='CMFPlomino_label_FieldValidation',
            description_msgid='CMFPlomino_help_FieldValidation',
            i18n_domain='CMFPlomino',
        ),
    ),
    BooleanField(
        name='ToBeIndexed',
        default="0",
        widget=BooleanField._properties['widget'](
            label="Add to index",
            description="The field will be searchable",
            label_msgid='CMFPlomino_label_FieldIndex',
            description_msgid='CMFPlomino_help_FieldIndex',
            i18n_domain='CMFPlomino',
        ),
    ),
),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

PlominoField_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
def get_field_types():
    field_types = FIELD_TYPES
    for plugin_field in component.getUtilitiesFor(interfaces.IPlominoField):
        params = plugin_field[1].plomino_field_parameters
        field_types[str(plugin_field[0])] = [params['label'], params['index_type']]
    return field_types

##/code-section after-schema

class PlominoField(BaseContent, BrowserDefaultMixin):
    """
    """
    security = ClassSecurityInfo()
    implements(interfaces.IPlominoField)

    meta_type = 'PlominoField'
    _at_rename_after_creation = False

    schema = PlominoField_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    security.declarePublic('validateFormat')
    def validateFormat(self, submittedValue):
        """check if submitted value match the field expected format
        """
        adapt = self.getSettings()
        return adapt.validate(submittedValue)

    security.declarePublic('processInput')
    def processInput(self, submittedValue, doc, process_attachments):
        """process submitted value according the field type
        """
        fieldtype = self.getFieldType()
        fieldname = self.id
        adapt = self.getSettings()
        if fieldtype=="ATTACHMENT" and process_attachments:
            if isinstance(submittedValue, FileUpload):
                current_files=doc.getItem(fieldname)
                if current_files=='':
                    current_files={}
                (new_file, contenttype) = doc.setfile(submittedValue)
                if new_file is not None:
                    current_files[new_file]=contenttype
                v=current_files
        else:
            v = adapt.processInput(submittedValue)
        return v

    security.declareProtected(READ_PERMISSION, 'getFieldRender')
    def getFieldRender(self, form, doc, editmode, creation=False, request=None):
        """Rendering the field
        """
        mode = self.getFieldMode()
        fieldname = self.id
        if doc is None:
            target = form
        else:
            target = doc

        adapt = self.getSettings()
        fieldvalue = adapt.getFieldValue(form, doc, editmode, creation, request)

        # get the rendering template
        pt=None
        if mode=="EDITABLE" and editmode:
            templatemode="Edit"
            # if custom template, use it
            if not self.getFieldEditTemplate()=="":
                pt=getattr(self.resources, self.getFieldEditTemplate()).__of__(self)
        else:
            templatemode="Read"
            # if custom template, use it
            if not self.getFieldReadTemplate()=="":
                pt=getattr(self.resources, self.getFieldReadTemplate()).__of__(self)

        # if no custom template provided, get the template associated to the field type
        if pt is None:
            if templatemode=="Read" and hasattr(adapt, 'read_template'):
                pt = adapt.read_template
            elif templatemode=="Edit" and hasattr(adapt, 'edit_template'):
                pt = adapt.edit_template
            else:
                fieldType = self.FieldType
                pt=self.getRenderingTemplate(fieldType+"Field"+templatemode)
                if pt is None:
                    pt=self.getRenderingTemplate("DefaultField"+templatemode)

        selection = self.getSettings().getSelectionList(target)

        try:
            return pt(fieldname=fieldname,
                fieldvalue=fieldvalue,
                selection=selection,
                field=self,
                doc=target
                )
        except Exception, e:
            self.traceRenderingErr(e, self)
            return ""

    security.declarePublic('getParentDatabase')
    def getParentDatabase(self):
        """Get the database containing this field
        """
        return self.getParentNode().getParentDatabase()

    security.declarePublic('at_post_edit_script')
    def at_post_edit_script(self):
        """post edit
        """
        self.cleanFormulaScripts("field_"+self.getParentNode().id+"_"+self.id)
        db = self.getParentDatabase()
        if self.getToBeIndexed() and not db.DoNotReindex:
            db.getIndex().createFieldIndex(self.id, self.getFieldType())

    security.declarePublic('at_post_create_script')
    def at_post_create_script(self):
        """post create
        """
        db = self.getParentDatabase()
        if self.getToBeIndexed() and not db.DoNotReindex:
            db.getIndex().createFieldIndex(self.id, self.getFieldType())

    security.declarePublic('getSettings')
    def getSettings(self, key=None):
        """
        """
        fieldclass = component.queryUtility(interfaces.IPlominoField, self.FieldType, None)
        if fieldclass is None:
            if hasattr(fields, self.FieldType.lower()):
                fieldinterface = getattr(getattr(fields, self.FieldType.lower()), "I"+self.FieldType.capitalize()+"Field")
            else:
                fieldinterface = getattr(getattr(fields, "base"), "IBaseField")
        else:
            fieldinterface = fieldclass.plomino_field_parameters['interface']

        if key is None:
            return fieldinterface(self)
        else:
            return getattr(fieldinterface(self), key, None)

    def type_vocabulary(self):
        ALL_FIELD_TYPES = get_field_types()
        l = [[f, ALL_FIELD_TYPES[f][0]] for f in ALL_FIELD_TYPES.keys()]
        l.sort(key=lambda f:f[1])
        return l

registerType(PlominoField, PROJECTNAME)
# end of class PlominoField

##code-section module-footer #fill in your manual code here
##/code-section module-footer






# -*- coding: utf-8 -*-
#
# File: PlominoDocument.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# Generator: ArchGenXML Version 2.0
#            http://plone.org/products/archgenxml
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from zope.interface import implements
import interfaces
from Products.ATContentTypes.content.folder import ATFolder
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import normalizeString

from exceptions import PlominoScriptException
from Products.CMFPlomino.config import *

from interfaces import *
from Products.Archetypes.public import *
from AccessControl import Unauthorized
from time import strptime
from DateTime import DateTime
from zope import event
from Products.Archetypes.event import ObjectEditedEvent
from zope.component import queryUtility

# Import conditionally, so we don't introduce a hard dependency
try:
    from plone.i18n.normalizer.interfaces import IUserPreferredURLNormalizer
    from plone.i18n.normalizer.interfaces import IURLNormalizer
    URL_NORMALIZER = True
except ImportError:
    URL_NORMALIZER = False
    
from PlominoUtils import DateToString, StringToDate, sendMail, asUnicode, asList
from OFS.Image import File
from ZPublisher.HTTPRequest import FileUpload
try:
    from iw.fss.FileSystemStorage import FileSystemStorage, FSSFileInfo
except Exception, e:
    pass
try:
    from plone.app.blob.field import BlobWrapper
    from plone.app.blob.utils import guessMimetype
    HAS_BLOB = True
except Exception, e:
    HAS_BLOB = False

schema = Schema((

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

PlominoDocument_schema = getattr(ATFolder, 'schema', Schema(())).copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class PlominoDocument(ATFolder):
    """
    """
    security = ClassSecurityInfo()
    implements(interfaces.IPlominoDocument)

    meta_type = 'PlominoDocument'
    _at_rename_after_creation = False

    schema = PlominoDocument_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    security.declarePublic('__init__')
    def __init__(self,oid,**kw):
        """initialization
        """
        ATFolder.__init__(self, oid, **kw)
        self.items={}
        self.plomino_modification_time = DateTime().toZone('UTC')

    security.declarePublic('checkBeforeOpenDocument')
    def checkBeforeOpenDocument(self):
        """check read permission and open view  NOTE: if READ_PERMISSION
        set on the 'view' action itself, it causes error 'maximum
        recursion depth exceeded' if user hasn't permission
        """
        if self.isReader():
            return self.OpenDocument()
        else:
            raise Unauthorized, "You cannot read this content"

    def doc_path(self):
        #db = self.getParentDatabase()
        #return db.getPhysicalPath() + (self.id,)
        # here we return actual path, we do not hide plomino_documents anymore
        return self.getPhysicalPath()

    def doc_url(self):
        """ return valid and nice url:
        - hide plomino_documents
        - use physicalPathToURL if REQUEST available
        """
        path = self.doc_path()
        short_path = [p for p in path if p!="plomino_documents"]
        if hasattr(self, "REQUEST"):
            return self.REQUEST.physicalPathToURL(short_path)
        else:
            return "/".join(short_path)

    security.declarePublic('setItem')
    def setItem(self,name,value):
        """
        """
        items = self.items
        items[name] = value
        self.items = items
        self.plomino_modification_time = DateTime().toZone('UTC')

    security.declarePublic('getItem')
    def getItem(self,name, default=''):
        """
        """
        if(self.items.has_key(name)):
            return self.items[name]
        else:
            return default

    security.declarePublic('hasItem')
    def hasItem(self,name):
        """
        """
        return self.items.has_key(name)

    security.declarePublic('removeItem')
    def removeItem(self,name):
        """
        """
        if(self.items.has_key(name)):
            items = self.items
            del items[name]
            self.items = items

    security.declarePublic('getItems')
    def getItems(self):
        """
        """
        return self.items.keys()

    security.declarePublic('getItemClassname')
    def getItemClassname(self,name):
        """
        """
        return self.getItem(name).__class__.__name__

    security.declarePublic('getLastModified')
    def getLastModified(self,asString=False):
        """
        """
        if not hasattr(self, 'plomino_modification_time'):
            self.plomino_modification_time = self.bobobase_modification_time().toZone('UTC')
        if asString:
            return str(self.plomino_modification_time)
        else:
            return self.plomino_modification_time

    security.declarePublic('getRenderedItem')
    def getRenderedItem(self, itemname, form=None, formid=None, convertattachments=False):
        """ return the item value rendered according the field defined in the given form
        (use default doc form if None)
        """
        result = ''
        db = self.getParentDatabase()
        if not form:
            if not formid:
                form = self.getForm()
            else:
                form = db.getForm(formid)
        if form:
            field = form.getFormField(itemname)
            if field:
                result = field.getFieldRender(form, self, False)
                if field.getFieldType()=='ATTACHMENT' and convertattachments:
                    result = result + ' ' + db.getIndex().convertFileToText(self,itemname)
                return result

        return result

    security.declarePublic('computeItem')
    def computeItem(self, itemname, form=None, formid=None, store=True):
        """ return the item value according the formula of the field defined in
        the given form (use default doc form if None)
        and store the value in the doc (if store=True)
        """
        result = None
        db = self.getParentDatabase()
        if not form:
            if not formid:
                form = self.getForm()
            else:
                form = db.getForm(formid)
        if form:
            result = form.computeFieldValue(itemname, self)
            if store:
                self.setItem(itemname, result)
        return result

    security.declarePublic('getParentDatabase')
    def getParentDatabase(self):
        """
        """
        parent = self.getParentNode()
        if parent.id == "plomino_documents":
            return parent.getParentNode()
        else:
            return parent

    security.declarePublic('getPlominoReaders')
    def getPlominoReaders(self):
        """
        """
        if self.hasItem('Plomino_Readers'):
            return asList(self.Plomino_Readers)
        else:
            return ['*']

    security.declarePublic('isReader')
    def isReader(self):
        """
        """
        return self.getParentDatabase().isCurrentUserReader(self)
        
    security.declarePublic('isAuthor')
    def isAuthor(self):
        """
        """
        return self.getParentDatabase().isCurrentUserAuthor(self)

    security.declareProtected(REMOVE_PERMISSION, 'delete')
    def delete(self,REQUEST=None):
        """delete the current doc
        """
        db = self.getParentDatabase()
        db.deleteDocument(self)
        if not REQUEST is None:
            return_url = REQUEST.get('returnurl')
            REQUEST.RESPONSE.redirect(return_url)

    security.declareProtected(EDIT_PERMISSION, 'saveDocument')
    def saveDocument(self,REQUEST, creation=False):
        """save a document using the form submitted content
        """
        db = self.getParentDatabase()
        form = db.getForm(REQUEST.get('Form'))

        errors=form.validateInputs(REQUEST, doc=self)
        if len(errors)>0:
            return form.notifyErrors(errors)

        self.setItem('Form', form.getFormName())

        # process editable fields (we read the submitted value in the request)
        form.readInputs(self, REQUEST, process_attachments=True)

        # refresh computed values, run onSave, reindex
        self.save(form, creation)

        redirect = self.getItem("plominoredirecturl")
        if redirect=='':
            redirect = self.absolute_url()
        REQUEST.RESPONSE.redirect(redirect)


    security.declareProtected(EDIT_PERMISSION, 'refresh')
    def refresh(self, form=None):
        """ re-compute fields and re-index document
        (onSave event is not called, and authors are not updated 
        """
        self.save(form, creation=False, refresh_index=True, asAuthor=False, onSaveEvent=False)

    security.declareProtected(EDIT_PERMISSION, 'save')
    def save(self, form=None, creation=False, refresh_index=True, asAuthor=True, onSaveEvent=True):
        """refresh values according form, and reindex the document
        """
        # we process computed fields (refresh the value)
        # TODO: manage computed fields dependencies
        if form is None:
            form = self.getForm()
        else:
            self.setItem('Form', form.getFormName())

        db=self.getParentDatabase()
        if form:
            for f in form.getFormFields(includesubforms=True, doc=self):
                mode = f.getFieldMode()
                fieldname = f.id
                if mode in ["COMPUTED", "COMPUTEDONSAVE"] or (mode=="CREATION" and creation):
                    result = form.computeFieldValue(fieldname, self)
                    self.setItem(fieldname, result)
                else:
                    # computed for display field are not stored
                    pass

            # compute the document title
            try:
                result = self.runFormulaScript("form_"+form.id+"_title", self, form.DocumentTitle)
            except PlominoScriptException, e:
                e.reportError('Title formula failed')
                result = ""
            if not result:
                result = form.Title()
            self.setTitle(result)

            # update the document id
            if creation and form.getDocumentId():
                self._renameAfterCreation()
            
        # update the Plomino_Authors field with the current user name
        if asAuthor:
            authors = self.getItem('Plomino_Authors')
            name = db.getCurrentUser().getUserName()
            if authors == '':
                authors = []
                authors.append(name)
            elif name in authors:
                pass
            else:
                authors.append(name)
            self.setItem('Plomino_Authors', authors)

        # execute the onSaveDocument code of the form
        if form and onSaveEvent:
            try:
                self.runFormulaScript("form_"+form.id+"_onsave", self, form.onSaveDocument)
            except PlominoScriptException, e:
                if self.REQUEST:
                    e.reportError('Document has been saved but onSave event failed.')
                    doc_path = self.REQUEST.physicalPathToURL(self.doc_path())
                    self.REQUEST.RESPONSE.redirect(doc_path)

        if refresh_index:
            # update index
            db.getIndex().indexDocument(self)
            # update portal_catalog
            if db.getIndexInPortal():
                self.reindexObject()
            event.notify(ObjectEditedEvent(self))

    security.declareProtected(READ_PERMISSION, 'openWithForm')
    def openWithForm(self,form,editmode=False):
        """display the document using the given form's layout - first,
        check if the user has proper access rights
        """
        db = self.getParentDatabase()
        if editmode:
            if not db.isCurrentUserAuthor(self):
                raise Unauthorized, "You cannot edit this document."

        # execute the onOpenDocument code of the form
        valid = ''
        try:
            if not form.getOnOpenDocument()=="":
                #RunFormula(self, form.getOnOpenDocument())
                valid = self.runFormulaScript("form_"+form.id+"_onopen", self, form.onOpenDocument)
        except PlominoScriptException, e:
            e.reportError('onOpen event failed')

        if not valid:
            # we use the specified form's layout
            request = getattr(self, 'REQUEST', None)
            if not request:
                import sys
                from ZPublisher.HTTPResponse import HTTPResponse
                from ZPublisher.HTTPRequest import HTTPRequest
                response = HTTPResponse(stdout=sys.stdout)
                env = {'SERVER_NAME':'fake_server',
                       'SERVER_PORT':'80',
                       'REQUEST_METHOD':'GET'}
                request = HTTPRequest(sys.stdin, env, response)
            html_content = form.displayDocument(self,
                                editmode=editmode,
                                request=request)
        else:
            html_content = valid

        plone_tools = getToolByName(db, 'plone_utils')
        encoding = plone_tools.getSiteEncoding()
        html_content = html_content.encode(encoding) 

        return html_content


    security.declareProtected(EDIT_PERMISSION, 'editWithForm')
    def editWithForm(self,form):
        """
        """
        return self.openWithForm(form, True)

    security.declarePublic('send')
    def send(self,recipients,title,form=None):
        """Send current doc by mail
        """
        db = self.getParentDatabase()
        if form is None:
            form = self.getForm()
        message = self.openWithForm(form)
        sendMail(db, recipients, title, message)

    security.declarePublic('getForm')
    def getForm(self):
        """by default, we use the form corresponding to the Form item value
        but it might be forced to a different form by passing the form id as
        request parameter, or by evaluating the parent view form formula
        """
        formname = None
        if hasattr(self, 'REQUEST'):
            formname = self.REQUEST.get("openwithform", None)
        if not formname:
            if hasattr(self, 'evaluateViewForm'):
                formname = self.evaluateViewForm(self)
        if not formname:
            formname = self.getItem('Form')

        return self.getParentDatabase().getForm(formname)

    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self,item):
        """after cloning
        """
        # changed from BaseFolder to ATFolder because now inherits fron ATFolder
        ATFolder.manage_afterClone(self, item)

    security.declarePublic('__getattr__')
    def __getattr__(self,name):
        """Overloads getattr to return item values as attibutes
        """
        if(self.items.has_key(name)):
            return self.items[name]
        else:
            raise AttributeError, name
            #return BaseObject.__getattr__(self, name)

    security.declareProtected(READ_PERMISSION, 'isSelectedInView')
    def isSelectedInView(self,viewname):
        """
        """
        db = self.getParentDatabase()
        v = db.getView(viewname)
        result = False
        if v:
            try:
                #result = RunFormula(self, v.SelectionFormula())
                result = self.runFormulaScript("view_"+v.id+"_selection", self, v.SelectionFormula)
            except PlominoScriptException, e:
                e.reportError('%s view selection formula failed' % viewname)
        return result

    security.declareProtected(READ_PERMISSION, 'computeColumnValue')
    def computeColumnValue(self,viewname,columnname):
        """
        """
        db = self.getParentDatabase()
        v = db.getView(viewname)
        try:
            c = v.getColumn(columnname)
            #result = RunFormula(self, c.Formula())
            result = self.runFormulaScript("column_"+v.id+"_"+c.id+"_formula", self, c.Formula)
        except PlominoScriptException, e:
            e.reportError('"%s" column formula failed in %s view' % (c.Title(), viewname))
            result = None
        return result

    security.declareProtected(EDIT_PERMISSION, 'deleteAttachment')
    def deleteAttachment(self, REQUEST=None, fieldname=None, filename=None):
        """remove file object and update corresponding item value
        """
        if REQUEST is not None:
            fieldname=REQUEST.get('field')
            filename=REQUEST.get('filename')
        if fieldname is not None and filename is not None:
            self.deletefile(filename)
            current_files=self.getItem(fieldname)
            if current_files.has_key(filename):
                del current_files[filename]
                self.setItem(fieldname, current_files)
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url()+"/EditDocument")

    security.declarePublic('SearchableText')
    def SearchableText(self):
        values = []
        index_attachments=self.getParentDatabase().getIndexAttachments()
        form = self.getForm()

        for itemname in self.items.keys():
            item_value = self.getItem(itemname)
            if type(item_value) is list: 
                for v in item_value:
                    if type(v) is list:
                        values = values + [asUnicode(k) for k in v]
                    else:
                        values.append(asUnicode(v))
            else:
                values.append(asUnicode(item_value))
            # if selection or attachment field, we try to index rendered values too
            try:
                if form:
                    field = form.getFormField(itemname)
                    if field and field.getFieldType() in ["SELECTION", "ATTACHMENT"]:
                        v = asUnicode(self.getRenderedItem(itemname,form=form, convertattachments=index_attachments))
                        if v:
                            values.append(v)
            except:
                pass
        return ' '.join(values)

    security.declareProtected(READ_PERMISSION, 'getfile')
    def getfile(self, filename=None, REQUEST=None):
        """
        """
        fss = self.getParentDatabase().getStorageAttachments()
        if REQUEST is not None:
            filename = REQUEST.get('filename')
        if filename is not None:
            if fss:
                storage = FileSystemStorage()
                file_obj = storage.get(filename, self)
            else:
                #file_obj = getattr(self, filename)
                file_obj = self[filename]
            if REQUEST is None:
                return file_obj
            else:
                REQUEST.RESPONSE.setHeader('content-type', file_obj.getContentType())
                REQUEST.RESPONSE.setHeader("Content-Disposition", "inline; filename="+filename)
                if fss:
                    return file_obj.getData()
                else:
                    return file_obj.data
        else:
            return None

    security.declareProtected(READ_PERMISSION, 'getFilenames')
    def getFilenames(self):
        """
        """
        fss = self.getParentDatabase().getStorageAttachments()
        if fss:
            return [o.getTitle() for o in self.__dict__.values() if isinstance(o, FSSFileInfo)]
        else:
            #return [o.getId() for o in self.getChildNodes() if isinstance(o, File)]
            if HAS_BLOB:
                return [id for id in self.objectIds() if isinstance(self[id], BlobWrapper)]
            else:
                return [id for id in self.objectIds() if isinstance(self[id], File)]

    security.declareProtected(EDIT_PERMISSION, 'setfile')
    def setfile(self, submittedValue, filename='', overwrite=False, contenttype=''):
        """
        """
        if filename=='':
            filename=submittedValue.filename
        if filename!='':
            if """\\""" in filename:
                filename=filename.split("\\")[-1]
            filename = ".".join([normalizeString(s, encoding='utf-8') for s in filename.split('.')])
            if filename in self.objectIds():
                if overwrite:
                    self.deletefile(filename)
                else:
                    return ("ERROR: "+filename+" already exists", "")
            else:
                if(self.getParentDatabase().getStorageAttachments()==True):
                    tmpfile=File(filename, filename, submittedValue)
                    storage = FileSystemStorage();
                    storage.set(filename, self, tmpfile);
                    contenttype=storage.get(filename,self).getContentType()
                elif HAS_BLOB:
                    if isinstance(submittedValue, FileUpload) or type(submittedValue) == file:
                        submittedValue.seek(0)
                        contenttype = guessMimetype(submittedValue, filename)
                        submittedValue = submittedValue.read()
                    try:
                        blob = BlobWrapper(contenttype)
                    except:
                        # BEFORE PLONE 4.0.1
                        blob = BlobWrapper()
                    file_obj = blob.getBlob().open('w')
                    file_obj.write(submittedValue)
                    file_obj.close()
                    blob.setFilename(filename)
                    blob.setContentType(contenttype)
                    self._setObject(filename, blob)
                else:
                    self.manage_addFile(filename, submittedValue)
                    contenttype=self[filename].getContentType()
                return (filename, contenttype)
        else:
            return (None, "")

    security.declareProtected(EDIT_PERMISSION, 'deletefile')
    def deletefile(self, filename):
        """
        """
        if(self.getParentDatabase().getStorageAttachments()):
            storage = FileSystemStorage();
            storage.unset(filename, self);
        else:
            if filename in self.objectIds():
                self.manage_delObjects(filename)

    security.declarePublic('isNewDocument')
    def isNewDocument(self):
        """
        """
        # when the context is an actual document, it is necessarily not new
        return False

    security.declarePublic('isEditMode')
    def isEditMode(self):
        """
        """
        # if REQUEST exists, test the current command
        if hasattr(self, 'REQUEST'):
            command=self.REQUEST.URL.split('/')[-1].lower()
            return command in ['editdocument', 'edit']
        else:
            return False

    def generateNewId(self):
        """ compute the id using the Document id formula
        (overwrite Archetypes generateNewId, and provides same behaviour, 
        but use Document id formula instead of title)
        """
        form = self.getForm()
        if not form:
            return None

        if not form.getDocumentId():
            return None

        result = None
        try:
            result = self.runFormulaScript("form_"+form.id+"_docid", self, form.DocumentId)
        except PlominoScriptException, e:
            e.reportError('Document id formula failed')

        if not result:
            return None

        # Don't do anything without the plone.i18n package
        if not URL_NORMALIZER:
            return None

        if not isinstance(result, unicode):
            charset = self.getCharset()
            result = unicode(result, charset)

        request = getattr(self, 'REQUEST', None)
        if request is not None:
            return IUserPreferredURLNormalizer(request).normalize(result)

        return queryUtility(IURLNormalizer).normalize(result)
    
registerType(PlominoDocument, PROJECTNAME)
# end of class PlominoDocument

##code-section module-footer #fill in your manual code here
class TemporaryDocument(PlominoDocument):

    security = ClassSecurityInfo()

    def __init__(self, parent, form, REQUEST, real_doc=None):
        self._parent=parent
        if real_doc is not None:
            self.items=real_doc.items.copy()
            self.real_id=real_doc.id
        else:
            self.items={}
            self.real_id="TEMPDOC"
        self.setItem('Form', form.getFormName())
        form.readInputs(self, REQUEST)
        self.REQUEST=REQUEST

    security.declarePublic('getParentDatabase')
    def getParentDatabase(self):
        return self._parent

    security.declarePublic('isEditMode')
    def isEditMode(self):
        """
        """
        return True

    security.declarePublic('isNewDocument')
    def isNewDocument(self):
        """
        """
        if self.real_id=="TEMPDOC":
            return True
        else:
            return False

    security.declarePublic('id')
    @property
    def id(self):
        """
        """
        return self.real_id

##/code-section module-footer



